@@ -1,4 +1,4 @@
-/*	$calcurse: apoint.c,v 1.23 2008/04/19 21:05:15 culot Exp $	*/
+/*	$calcurse: apoint.c,v 1.28 2008/12/15 20:02:00 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -32,12 +32,13 @@
 #include "i18n.h"
 #include "vars.h"
 #include "event.h"
-#include "apoint.h"
 #include "day.h"
 #include "custom.h"
 #include "notify.h"
 #include "recur.h"
+#include "keys.h"
 #include "calendar.h"
+#include "apoint.h"
 
 apoint_llist_t *alist_p;
 static int hilt = 0;
@@ -150,7 +151,7 @@ apoint_add (void)
 	  else if (check_time (item_time) != 1)
 	    {
 	      status_mesg (format_message_1, enter_str);
-	      wgetch (win[STA].p);
+	      (void)wgetch (win[STA].p);
 	    }
 	  else
 	    sscanf (item_time, "%u:%u", &heures, &minutes);
@@ -174,7 +175,7 @@ apoint_add (void)
 	  else if (check_time (item_time) == 0)
 	    {
 	      status_mesg (format_message_2, enter_str);
-	      wgetch (win[STA].p);
+	      (void)wgetch (win[STA].p);
 	    }
 	  else
 	    {
@@ -228,7 +229,6 @@ apoint_delete (conf_t *conf, unsigned *nb_events, unsigned *nb_apoints)
 {
   char *choices = "[y/n] ";
   char *del_app_str = _("Do you really want to delete this item ?");
-  const char *errmsg = _("FATAL ERROR in apoint_delete: no such type\n");
   long date;
   int nb_items = *nb_apoints + *nb_events;
   bool go_for_deletion = false;
@@ -269,11 +269,9 @@ apoint_delete (conf_t *conf, unsigned *nb_events, unsigned *nb_apoints)
 	      to_be_removed = 3;
 	    }
 	  else if (deleted_item_type == 0)
-	    {
-	      to_be_removed = 0;
-	    }
+            return;
 	  else
-	    ierror (errmsg, IERROR_FATAL);
+            EXIT (_("no such type"));
 	  /* NOTREACHED */
 
 	  if (hilt > 1)
@@ -381,12 +379,8 @@ apoint_scan (FILE *f, struct tm start, struct tm end, char state, char *note)
 
   tstart = mktime (&start);
   tend = mktime (&end);
-  if (tstart == -1 || tend == -1 || tstart > tend)
-    {
-      fputs (_("FATAL ERROR in apoint_scan: date error in the appointment\n"),
-	     stderr);
-      exit (EXIT_FAILURE);
-    }
+  EXIT_IF (tstart == -1 || tend == -1 || tstart > tend,
+           _("date error in appointment"));
   return (apoint_new (buf, note, tstart, tend - tstart, state));
 }
 
@@ -407,9 +401,9 @@ apoint_get (long day, int pos)
 	  n++;
 	}
     }
-  /* NOTREACHED */
-  fputs (_("FATAL ERROR in apoint_get: no such item\n"), stderr);
-  exit (EXIT_FAILURE);
+  EXIT (_("item not found"));
+  return 0;
+  /* NOTREACHED */  
 }
 
 void
@@ -429,7 +423,10 @@ apoint_delete_bynum (long start, unsigned num, erase_flag_e flag)
 	  if (n == num)
 	    {
 	      if (flag == ERASE_FORCE_ONLY_NOTE)
-		erase_note (&i->note, flag);
+                {
+                  erase_note (&i->note, flag);
+                  pthread_mutex_unlock (&(alist_p->mutex));
+                }
 	      else
 		{
 		  if (notify_bar ())
@@ -451,8 +448,7 @@ apoint_delete_bynum (long start, unsigned num, erase_flag_e flag)
   pthread_mutex_unlock (&(alist_p->mutex));
 
   /* NOTREACHED */
-  ierror (_("FATAL ERROR in apoint_delete_bynum: no such appointment"),
-	  IERROR_FATAL);
+  EXIT (_("no such appointment"));
 }
 
 /*
@@ -532,7 +528,7 @@ apoint_check_next (struct notify_app_s *app, long start)
 	  if (i->start > start)
 	    {
 	      app->time = i->start;
-	      app->txt = mycpy (i->mesg);
+	      app->txt = strdup (i->mesg);
 	      app->state = i->state;
 	      app->got_app = 1;
 	    }
@@ -612,8 +608,7 @@ apoint_switch_notify (void)
   pthread_mutex_unlock (&(alist_p->mutex));
 
   /* NOTREACHED */
-  ierror (_("FATAL ERROR in apoint_switch_notify: no such appointment"),
-	  IERROR_FATAL);
+  EXIT (_("no such appointment"));
 }
 
 /* Updates the Appointment panel */
