@@ -1,4 +1,4 @@
-/*	$calcurse: calcurse.c,v 1.68 2008/09/21 08:06:43 culot Exp $	*/
+/*	$calcurse: calcurse.c,v 1.73 2008/12/20 19:27:31 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -41,7 +41,7 @@
 #include "todo.h"
 #include "args.h"
 #include "notify.h"
-
+#include "keys.h"
 
 /*
  * Calcurse is a text-based personal organizer which helps keeping track
@@ -55,7 +55,7 @@ main (int argc, char **argv)
 {
   conf_t conf;
   day_items_nb_t inday;
-  int ch, background, foreground;
+  int background, foreground;
   int non_interactive;
   int no_data_file = 1;
   int sav_hilt_app = 0;
@@ -133,6 +133,7 @@ main (int argc, char **argv)
     }
 
   vars_init (&conf);
+  keys_init ();
   wins_init ();
   wins_slctd_init ();
   notify_init_bar ();
@@ -146,6 +147,7 @@ main (int argc, char **argv)
   no_data_file = io_check_data_files ();
   custom_load_conf (&conf, background);
   erase_status_bar ();
+  io_load_keys (conf.pager);
   io_load_todo ();
   io_load_app ();
   wins_reinit ();
@@ -161,23 +163,25 @@ main (int argc, char **argv)
   /* User input */
   for (;;)
     {
+      int key;
+      
       do_update = true;
-      ch = wgetch (win[STA].p);
-      switch (ch)
+      key = keys_getch (win[STA].p);
+      switch (key)
 	{
 	case ERR:
 	  do_update = false;
 	  break;
 
-	case CTRL ('R'):
+	case KEY_GENERIC_REDRAW:
 	case KEY_RESIZE:
 	  do_update = false;
 	  wins_reset ();
 	  break;
 
-	case 9:		/* The TAB key was hit. */
+	case KEY_GENERIC_CHANGE_VIEW:
 	  reset_status_page ();
-	  /* Save previously highlighted event. */
+	  /* Need to save the previously highlighted event. */
 	  switch (wins_slctd ())
 	    {
 	    case TOD:
@@ -214,17 +218,15 @@ main (int argc, char **argv)
 	    }
 	  break;
 
-	case 'O':
-	case 'o':
+        case KEY_GENERIC_OTHER_CMD:
 	  other_status_page (wins_slctd ());
 	  break;
 
-        case CTRL ('G'):
-	case 'G':
-	case 'g':		/* Goto function */
+        case KEY_GENERIC_GOTO:
+        case KEY_GENERIC_GOTO_TODAY:
 	  erase_status_bar ();
 	  calendar_set_current_date ();
-          if (ch == CTRL ('G'))
+          if (key == KEY_GENERIC_GOTO_TODAY)
             calendar_goto_today ();
           else
             calendar_change_day (conf.input_datefmt);
@@ -232,21 +234,19 @@ main (int argc, char **argv)
 	  day_changed = true;
 	  break;
 
-	case 'V':
-	case 'v':		/* View function */
+        case KEY_VIEW_ITEM:
 	  if ((wins_slctd () == APP) && (apoint_hilt () != 0))
 	    day_popup_item ();
 	  else if ((wins_slctd () == TOD) && (todo_hilt () != 0))
 	    item_in_popup (NULL, NULL, todo_saved_mesg (), _("To do :"));
 	  break;
 
-	case 'C':
-	case 'c':		/* Configuration menu */
+        case KEY_GENERIC_CONFIG_MENU:
 	  erase_status_bar ();
-	  config_bar ();
-	  while ((ch = wgetch (win[STA].p)) != 'q')
+	  custom_config_bar ();
+	  while ((key = wgetch (win[STA].p)) != 'q')
 	    {
-	      switch (ch)
+	      switch (key)
 		{
 		case 'C':
 		case 'c':
@@ -262,7 +262,7 @@ main (int argc, char **argv)
 		  break;
 		case 'L':
 		case 'l':
-		  layout_config ();
+		  custom_layout_config ();
 		  break;
 		case 'G':
 		case 'g':
@@ -272,29 +272,32 @@ main (int argc, char **argv)
 		case 'n':
 		  notify_config_bar ();
 		  break;
+                case 'K':
+                case 'k':
+                  custom_keys_config ();
+                  break;
 		}
 	      wins_reset ();
 	      wins_update ();
 	      do_storage = true;
 	      erase_status_bar ();
-	      config_bar ();
+	      custom_config_bar ();
 	    }
 	  wins_update ();
 	  break;
 
-	case CTRL ('A'):	/* Add an app, whatever panel selected */
+        case KEY_GENERIC_ADD_APPT:
 	  apoint_add ();
 	  do_storage = true;
 	  break;
 
-	case CTRL ('T'):	/* Add a todo, whatever panel selected */
+        case KEY_GENERIC_ADD_TODO:
 	  todo_new_item ();
 	  if (todo_hilt () == 0 && todo_nb () == 1)
 	    todo_hilt_increase ();
 	  break;
 
-	case 'A':
-	case 'a':		/* Add an item */
+        case KEY_ADD_ITEM:
 	  switch (wins_slctd ())
 	    {
 	    case APP:
@@ -311,8 +314,7 @@ main (int argc, char **argv)
 	    }
 	  break;
 
-	case 'E':
-	case 'e':		/* Edit an existing item */
+        case KEY_EDIT_ITEM:
 	  if (wins_slctd () == APP && apoint_hilt () != 0)
 	    day_edit_item (&conf);
 	  else if (wins_slctd () == TOD && todo_hilt () != 0)
@@ -320,8 +322,7 @@ main (int argc, char **argv)
 	  do_storage = true;
 	  break;
 
-	case 'D':
-	case 'd':		/* Delete an item */
+        case KEY_DEL_ITEM:
 	  if (wins_slctd () == APP && apoint_hilt () != 0)
 	    apoint_delete (&conf, &inday.nb_events, &inday.nb_apoints);
 	  else if (wins_slctd () == TOD && todo_hilt () != 0)
@@ -329,24 +330,23 @@ main (int argc, char **argv)
 	  do_storage = true;
 	  break;
 
-	case 'R':
-	case 'r':
+        case KEY_REPEAT_ITEM:
 	  if (wins_slctd () == APP && apoint_hilt () != 0)
 	    recur_repeat_item (&conf);
 	  do_storage = true;
 	  break;
 
-	case '!':
+        case KEY_FLAG_ITEM:
 	  if (wins_slctd () == APP && apoint_hilt () != 0)
 	    apoint_switch_notify ();
 	  do_storage = true;
 	  break;
 
-	case '+':
-	case '-':
+        case KEY_RAISE_PRIORITY:
+        case KEY_LOWER_PRIORITY:
 	  if (wins_slctd () == TOD && todo_hilt () != 0)
 	    {
-	      todo_chg_priority (ch);
+	      todo_chg_priority (key);
 	      if (todo_hilt_pos () < 0)
 		todo_set_first (todo_hilt ());
 	      else if (todo_hilt_pos () >= win[TOD].h - 4)
@@ -354,9 +354,7 @@ main (int argc, char **argv)
 	    }
 	  break;
 
-	case 'N':
-	case 'n':
-	  /* Attach a note to an item, create it if necessary */
+        case KEY_EDIT_NOTE:
 	  if (wins_slctd () == APP && apoint_hilt () != 0)
 	    day_edit_note (conf.editor);
 	  else if (wins_slctd () == TOD && todo_hilt () != 0)
@@ -364,46 +362,42 @@ main (int argc, char **argv)
 	  do_storage = true;
 	  break;
 
-	case '>':
-	  /* View a note previously attached to an item */
+        case KEY_VIEW_NOTE:
 	  if (wins_slctd () == APP && apoint_hilt () != 0)
 	    day_view_note (conf.pager);
 	  else if (wins_slctd () == TOD && todo_hilt () != 0)
 	    todo_view_note (conf.pager);
 	  break;
 
-	case '?':		/* Online help system */
+        case KEY_GENERIC_HELP:
 	  status_bar ();
 	  help_screen ();
 	  break;
 
-	case 'S':
-	case 's':		/* Save function */
-	  io_save_cal (IO_MODE_INTERACTIVE, &conf);
+        case KEY_GENERIC_SAVE:
+	  io_save_cal (&conf);
 	  break;
 
-        case 'I':
-        case 'i':               /* Import function */
+        case KEY_GENERIC_IMPORT:
           erase_status_bar ();
-          io_import_data (IO_MODE_INTERACTIVE, IO_IMPORT_ICAL, &conf, NULL);
+          io_import_data (IO_IMPORT_ICAL, &conf, NULL);
           do_storage = true;
           break;
           
-	case 'X':
-	case 'x':		/* Export function */
+        case KEY_GENERIC_EXPORT:
           erase_status_bar ();
           io_export_bar ();
-          while ((ch = wgetch (win[STA].p)) != 'q')
+          while ((key = wgetch (win[STA].p)) != 'q')
 	    {
-	      switch (ch)
+	      switch (key)
 		{
 		case 'I':
 		case 'i':
-                  io_export_data (IO_MODE_INTERACTIVE, IO_EXPORT_ICAL, &conf);
+                  io_export_data (IO_EXPORT_ICAL, &conf);
 		  break;
 		case 'P':
 		case 'p':
-                  io_export_data (IO_MODE_INTERACTIVE, IO_EXPORT_PCAL, &conf);
+                  io_export_data (IO_EXPORT_PCAL, &conf);
 		  break;
 		}
 	      wins_reset ();
@@ -415,11 +409,9 @@ main (int argc, char **argv)
 	  wins_update ();
 	  break;
 
-        case KEY_RIGHT:
-	case ('L'):
-	case ('l'):
-	case CTRL ('L'):
-	  if (wins_slctd () == CAL || ch == CTRL ('L'))
+        case KEY_GENERIC_NEXT_DAY:
+        case KEY_MOVE_RIGHT:
+	  if (wins_slctd () == CAL || key == KEY_GENERIC_NEXT_DAY)
 	    {
 	      do_storage = true;
 	      day_changed = true;
@@ -427,11 +419,9 @@ main (int argc, char **argv)
 	    }
 	  break;
 
-        case KEY_LEFT:
-	case ('H'):
-	case ('h'):
-	case CTRL ('H'):
-	  if (wins_slctd () == CAL || ch == CTRL ('H'))
+        case KEY_GENERIC_PREV_DAY:
+        case KEY_MOVE_LEFT:
+	  if (wins_slctd () == CAL || key == KEY_GENERIC_PREV_DAY)
 	    {
 	      do_storage = true;
 	      day_changed = true;
@@ -439,60 +429,50 @@ main (int argc, char **argv)
 	    }
 	  break;
 
-        case KEY_UP:
-	case ('K'):
-	case ('k'):
-	case CTRL ('K'):
-	  if (wins_slctd () == CAL || ch == CTRL ('K'))
+        case KEY_GENERIC_PREV_WEEK:
+        case KEY_MOVE_UP:
+	  if (wins_slctd () == CAL || key == KEY_GENERIC_PREV_WEEK)
 	    {
 	      do_storage = true;
 	      day_changed = true;
 	      calendar_move (UP);
 	    }
-	  else
-	    {
-	      if ((wins_slctd () == APP) && (apoint_hilt () > 1))
-		{
-		  apoint_hilt_decrease ();
-		  apoint_scroll_pad_up (inday.nb_events);
-		}
-	      else if ((wins_slctd () == TOD) && (todo_hilt () > 1))
-		{
-		  todo_hilt_decrease ();
-		  if (todo_hilt_pos () < 0)
-		    todo_first_decrease ();
-		}
-	    }
-	  break;
+          else if ((wins_slctd () == APP) && (apoint_hilt () > 1))
+            {
+              apoint_hilt_decrease ();
+              apoint_scroll_pad_up (inday.nb_events);
+            }
+          else if ((wins_slctd () == TOD) && (todo_hilt () > 1))
+            {
+              todo_hilt_decrease ();
+              if (todo_hilt_pos () < 0)
+                todo_first_decrease ();
+            }
+          break;
 
-        case KEY_DOWN:
-	case ('J'):
-	case ('j'):
-	case CTRL ('J'):
-	  if (wins_slctd () == CAL || ch == CTRL ('J'))
+        case KEY_GENERIC_NEXT_WEEK:
+        case KEY_MOVE_DOWN:
+	  if (wins_slctd () == CAL || key == KEY_GENERIC_NEXT_WEEK)
 	    {
 	      do_storage = true;
 	      day_changed = true;
 	      calendar_move (DOWN);
 	    }
-	  else
-	    {
-	      if ((wins_slctd () == APP) &&
-		  (apoint_hilt () < inday.nb_events + inday.nb_apoints))
-		{
-		  apoint_hilt_increase ();
-		  apoint_scroll_pad_down (inday.nb_events, win[APP].h);
-		}
-	      if ((wins_slctd () == TOD) && (todo_hilt () < todo_nb ()))
-		{
-		  todo_hilt_increase ();
-		  if (todo_hilt_pos () == win[TOD].h - 4)
-		    todo_first_increase ();
-		}
-	    }
-	  break;
+          else if ((wins_slctd () == APP) &&
+              (apoint_hilt () < inday.nb_events + inday.nb_apoints))
+            {
+              apoint_hilt_increase ();
+              apoint_scroll_pad_down (inday.nb_events, win[APP].h);
+            }
+          else if ((wins_slctd () == TOD) && (todo_hilt () < todo_nb ()))
+            {
+              todo_hilt_increase ();
+              if (todo_hilt_pos () == win[TOD].h - 4)
+                todo_first_increase ();
+            }
+          break;
 
-        case '0':
+        case KEY_START_OF_WEEK:
           if (wins_slctd () == CAL)
 	    {
 	      do_storage = true;
@@ -501,7 +481,7 @@ main (int argc, char **argv)
 	    }
           break;
 
-        case '$':
+        case KEY_END_OF_WEEK:
           if (wins_slctd () == CAL)
 	    {
 	      do_storage = true;
@@ -510,16 +490,15 @@ main (int argc, char **argv)
 	    }
           break;
           
-	case ('Q'):		/* Quit calcurse :( */
-	case ('q'):
+        case KEY_GENERIC_QUIT:          
 	  if (conf.auto_save)
-	    io_save_cal (IO_MODE_INTERACTIVE, &conf);
+	    io_save_cal (&conf);
 
 	  if (conf.confirm_quit)
 	    {
 	      status_mesg (_(quit_message), choices);
-	      ch = wgetch (win[STA].p);
-	      if (ch == 'y')
+	      key = wgetch (win[STA].p);
+	      if (key == 'y')
 		exit_calcurse (EXIT_SUCCESS);
 	      else
 		{
