@@ -1,4 +1,4 @@
-/*	$calcurse: calendar.c,v 1.17 2008/08/10 09:24:46 culot Exp $	*/
+/*	$calcurse: calendar.c,v 1.20 2008/12/14 15:54:51 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -35,10 +35,11 @@
 #include "day.h"
 #include "apoint.h"
 #include "event.h"
-#include "calendar.h"
 #include "custom.h"
 #include "vars.h"
+#include "keys.h"
 #include "utils.h"
+#include "calendar.h"
 
 #ifndef M_PI
 #define	M_PI	  3.14159265358979323846
@@ -129,7 +130,7 @@ calendar_set_first_day_of_week (wday_e first_day)
       week_begins_on_monday = true;
       break;
     default:
-      fputs (_("ERROR in calendar_set_first_day_of_week\n"), stderr);
+      ERROR_MSG (_("ERROR setting first day of week"));
       week_begins_on_monday = false;
       /* NOTREACHED */
     }
@@ -387,7 +388,7 @@ calendar_change_day (int datefmt)
 	  if (wrong_day)
 	    {
 	      status_mesg (mesg_line1, mesg_line2);
-	      wgetch (win[STA].p);
+	      (void)wgetch (win[STA].p);
 	    }
 	}
     }
