@@ -1,4 +1,4 @@
-/*	$calcurse: custom.c,v 1.23 2008/09/21 08:06:43 culot Exp $	*/
+/*	$calcurse: custom.c,v 1.31 2008/12/20 19:27:31 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -26,12 +26,15 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <math.h>
 
 #include "custom.h"
 #include "i18n.h"
 #include "io.h"
 #include "utils.h"
+#include "keys.h"
 #include "apoint.h"
+#include "help.h"
 
 static struct attribute_s attr;
 
@@ -44,9 +47,8 @@ fill_config_var (char *string)
     return (false);
   else
     {
-      fputs (_("FATAL ERROR in fill_config_var: "
-	       "wrong configuration variable format.\n"), stderr);
-      return (EXIT_FAILURE);
+      EXIT (_("wrong configuration variable format."));
+      return false;
     }
 }
 
@@ -63,26 +65,16 @@ custom_load_color (char *color, int background)
   int i, len, color_num;
   char c[AWAITED_COLORS][BUFSIZ];
   int colr[AWAITED_COLORS];
-  const char *wrong_color_number =
-      _("FATAL ERROR in custom_load_color: wrong color number.\n");
-  const char *wrong_color_name =
-      _("FATAL ERROR in custom_load_color: wrong color name.\n");
-  const char *wrong_variable_format =
-      _("FATAL ERROR in custom_load_color: "
-	"wrong configuration variable format.\n");
 
   len = strlen (color);
-
   if (len > 1)
     {
       /* New version configuration */
       if (sscanf (color, "%s on %s", c[0], c[1]) != AWAITED_COLORS)
 	{
-	  fputs (_("FATAL ERROR in custom_load_color: "
-		   "missing colors in config file.\n"), stderr);
-	  exit (EXIT_FAILURE);
+          EXIT (_("missing colors in config file"));
 	  /* NOTREACHED */
-	};
+	}
 
       for (i = 0; i < AWAITED_COLORS; i++)
 	{
@@ -106,8 +98,7 @@ custom_load_color (char *color, int background)
 	    colr[i] = background;
 	  else
 	    {
-	      fputs (wrong_color_name, stderr);
-	      exit (EXIT_FAILURE);
+              EXIT (_("wrong color name"));
 	      /* NOTREACHED */
 	    }
 	}
@@ -148,15 +139,13 @@ custom_load_color (char *color, int background)
 	  init_pair (COLR_CUSTOM, COLOR_RED, COLR_BLUE);
 	  break;
 	default:
-	  fputs (wrong_color_number, stderr);
-	  exit (EXIT_FAILURE);
+          EXIT (_("wrong color number"));
 	  /* NOTREACHED */
 	}
     }
   else
     {
-      fputs (wrong_variable_format, stderr);
-      exit (EXIT_FAILURE);
+      EXIT (_("wrong configuration variable format"));
       /* NOTREACHED */
     }
 }
@@ -225,9 +214,9 @@ custom_load_conf (conf_t *conf, int background)
       status_mesg (mesg_line1, mesg_line2);
       wnoutrefresh (win[STA].p);
       doupdate ();
-      wgetch (win[STA].p);
+      keys_getch (win[STA].p);
     }
-  var = 0;
+  var = CUSTOM_CONF_NOVARIABLE;
   pthread_mutex_lock (&nbar->mutex);
   for (;;)
     {
@@ -308,9 +297,7 @@ custom_load_conf (conf_t *conf, int background)
 	  var = 0;
 	  break;
 	default:
-	  fputs (_("FATAL ERROR in custom_load_conf: "
-		   "configuration variable unknown.\n"), stderr);
-	  exit (EXIT_FAILURE);
+          EXIT (_("configuration variable unknown"));
 	  /* NOTREACHED */
 	}
 
@@ -351,7 +338,7 @@ custom_load_conf (conf_t *conf, int background)
 
 /* Draws the configuration bar */
 void
-config_bar (void)
+custom_config_bar (void)
 {
   int smlspc, spc;
 
@@ -364,6 +351,7 @@ config_bar (void)
   mvwprintw (win[STA].p, 0, 2 + spc, "L");
   mvwprintw (win[STA].p, 1, 2 + spc, "C");
   mvwprintw (win[STA].p, 0, 2 + 2 * spc, "N");
+  mvwprintw (win[STA].p, 1, 2 + 2 * spc, "K");  
   custom_remove_attr (win[STA].p, ATTR_HIGHEST);
 
   mvwprintw (win[STA].p, 0, 2 + smlspc, _("Exit"));
@@ -371,41 +359,178 @@ config_bar (void)
   mvwprintw (win[STA].p, 0, 2 + spc + smlspc, _("Layout"));
   mvwprintw (win[STA].p, 1, 2 + spc + smlspc, _("Color"));
   mvwprintw (win[STA].p, 0, 2 + 2 * spc + smlspc, _("Notify"));
-
+  mvwprintw (win[STA].p, 1, 2 + 2 * spc + smlspc, _("Keys"));
+  
   wnoutrefresh (win[STA].p);
   wmove (win[STA].p, 0, 0);
   doupdate ();
 }
 
+static void
+layout_selection_bar (void)
+{
+  binding_t quit    = {_("Exit"),     KEY_GENERIC_QUIT};
+  binding_t select  = {_("Select"),   KEY_GENERIC_SELECT};
+  binding_t up      = {_("Up"),       KEY_MOVE_UP};
+  binding_t down    = {_("Down"),     KEY_MOVE_DOWN};
+  binding_t left    = {_("Left"),     KEY_MOVE_LEFT};
+  binding_t right   = {_("Right"),    KEY_MOVE_RIGHT};
+  binding_t help    = {_("Help"),     KEY_GENERIC_HELP};
+  
+  binding_t *binding[] = {&quit, &select, &up, &down, &left, &right, &help};
+  int binding_size = sizeof (binding) / sizeof (binding[0]);
+
+  keys_display_bindings_bar (win[STA].p, binding, 0, binding_size);
+}
+
+#define NBLAYOUTS     8
+#define LAYOUTSPERCOL 2
+
+/* Used to display available layouts in layout configuration menu. */
+static void
+display_layout_config (window_t *lwin, int mark, int cursor, int need_reset)
+{
+#define CURSOR			(32 | A_REVERSE)
+#define MARK			88
+#define LAYOUTH                  5
+#define LAYOUTW                  9
+  char *box = "[ ]";
+  const int BOXSIZ = strlen (box);  
+  const int NBCOLS = NBLAYOUTS / LAYOUTSPERCOL;
+  const int COLSIZ = LAYOUTW + BOXSIZ + 1;
+  const int XSPC = (col - NBCOLS * COLSIZ) / (NBCOLS + 1);
+  const int XOFST = (col - NBCOLS * (XSPC + COLSIZ)) / 2;
+  const int YSPC = (row - 8 - LAYOUTSPERCOL * LAYOUTH) / (LAYOUTSPERCOL + 1);
+  const int YOFST = (row - LAYOUTSPERCOL * (YSPC + LAYOUTH)) / 2;  
+  enum {YPOS, XPOS, NBPOS};
+  int pos[NBLAYOUTS][NBPOS];
+  char *layouts[LAYOUTH][NBLAYOUTS] = {
+    {"+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+"},
+    {"|   | c |", "|   | t |", "| c |   |", "| t |   |", "|   | c |", "|   | a |", "| c |   |", "| a |   |"},
+    {"| a +---+", "| a +---+", "+---+ a |", "|---+ a |", "| t +---+", "| t +---+", "+---+ t |", "+---+ t |"},
+    {"|   | t |", "|   | c |", "| t |   |", "| c |   |", "|   | a |", "|   | c |", "| a |   |", "| c |   |"},
+    {"+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+", "+---+---+"}
+  };
+  int i;
+
+  for (i = 0; i < NBLAYOUTS; i++)
+    {
+      pos[i][YPOS] = YOFST + (i % LAYOUTSPERCOL) * (YSPC + LAYOUTH);
+      pos[i][XPOS] = XOFST + (i / LAYOUTSPERCOL) * (XSPC + COLSIZ);
+    }
+
+  if (need_reset)
+    {
+      char label[BUFSIZ];
+      
+      if (lwin->p != NULL)
+	delwin (lwin->p);
+      snprintf (label, BUFSIZ, _("CalCurse %s | layout configuration"),
+                VERSION);
+      custom_confwin_init (lwin, label);
+    }
+
+  for (i = 0; i < NBLAYOUTS; i++)
+    {
+      int j;
+      
+      mvwprintw (lwin->p, pos[i][YPOS] + 2, pos[i][XPOS], box);
+      if (i == mark)
+        custom_apply_attr (lwin->p, ATTR_HIGHEST);
+      for (j = 0; j < LAYOUTH; j++)
+        {
+          mvwprintw (lwin->p, pos[i][YPOS] + j, pos[i][XPOS] + BOXSIZ + 1,
+                     layouts[j][i]);
+        }
+      if (i == mark)
+        custom_remove_attr (lwin->p, ATTR_HIGHEST);
+    }
+  mvwaddch (lwin->p, pos[mark][YPOS] + 2, pos[mark][XPOS] + 1, MARK);
+  mvwaddch (lwin->p, pos[cursor][YPOS] + 2, pos[cursor][XPOS] + 1, CURSOR);
+  
+  layout_selection_bar ();
+  wnoutrefresh (win[STA].p);
+  wnoutrefresh (lwin->p);
+  doupdate ();
+  if (notify_bar ())
+    notify_update_bar ();
+}
+
 /* Choose the layout */
 void
-layout_config (void)
+custom_layout_config (void)
 {
-  int ch;
-  char *layout_mesg =
-    _("Pick the desired layout on next screen [press ENTER]");
-  char *choice_mesg =
-    _("('A'= Appointment panel, 'C'= calendar panel, 'T'= todo panel)");
-  char *layout_up_mesg =
-    _("    AC       AT       CA       TA       TC       TA       CT       AT");
-  char *layout_down_mesg =
-    _(" [1]AT    [2]AC    [3]TA    [4]CA    [5]TA    [6]TC    [7]AT    [8]CT");
+  scrollwin_t hwin;
+  window_t conf_win;
+  int ch, mark, cursor, need_reset;
+  char *help_text =
+    _("With this configuration menu, one can choose where panels will be\n"
+      "displayed inside calcurse screen. \n"
+      "It is possible to choose between eight different configurations.\n"
+      "\nIn the configuration representations, letters correspond to:\n\n"
+      "       'c' -> calendar panel\n\n"      
+      "       'a' -> appointment panel\n\n"
+      "       't' -> todo panel\n\n");
+  
+  need_reset = 1;  
+  conf_win.p = (WINDOW *)0;
+  cursor = mark = wins_layout () - 1;  
+  clear ();
+  display_layout_config (&conf_win, mark, cursor, need_reset);
 
-  status_mesg (layout_mesg, choice_mesg);
-  wgetch (win[STA].p);
-  status_mesg (layout_up_mesg, layout_down_mesg);
-  wnoutrefresh (win[STA].p);
-  doupdate ();
-  while ((ch = wgetch (win[STA].p)) != 'q')
+  while ((ch = keys_getch (win[STA].p)) != KEY_GENERIC_QUIT)
     {
-      if (ch <= '8' && ch >= '1')
+      need_reset = 0;
+      switch (ch)
 	{
-	  wins_set_layout (ch - '0');
-	  return;
+	case KEY_RESIZE:
+	  endwin ();
+	  refresh ();
+	  curs_set (0);
+	  need_reset = 1;
+	  break;
+        case KEY_GENERIC_HELP:
+          help_wins_init (&hwin, 0, 0,
+                          (notify_bar ()) ? row - 3 : row - 2, col);
+          mvwprintw (hwin.pad.p, 1, 0, "%s", help_text);
+          hwin.total_lines = 7;
+          wins_scrollwin_display (&hwin);
+          wgetch (hwin.win.p);
+          wins_scrollwin_delete (&hwin);
+          need_reset = 1;
+          break;
+	case KEY_GENERIC_SELECT:
+          mark = cursor;
+	  break;
+	case KEY_MOVE_DOWN:
+	  if (cursor % LAYOUTSPERCOL < LAYOUTSPERCOL - 1)
+	    cursor++;
+	  break;
+	case KEY_MOVE_UP:
+	  if (cursor % LAYOUTSPERCOL > 0)
+	    cursor--;
+	  break;
+	case KEY_MOVE_LEFT:
+	  if (cursor >= LAYOUTSPERCOL)
+	    cursor -= LAYOUTSPERCOL;
+	  break;
+	case KEY_MOVE_RIGHT:
+	  if (cursor < NBLAYOUTS - LAYOUTSPERCOL)
+	    cursor += LAYOUTSPERCOL;
+	  break;
+	case KEY_GENERIC_CANCEL:
+	  need_reset = 1;
+	  break;
 	}
+      display_layout_config (&conf_win, mark, cursor, need_reset);
     }
+  wins_set_layout (mark + 1);
+  delwin (conf_win.p);
 }
 
+#undef NBLAYOUTS
+#undef LAYOUTSPERCOL
+
 /* 
  * Create a configuration window and initialize status and notification bar 
  * (useful in case of window resize).
@@ -428,6 +553,26 @@ custom_confwin_init (window_t *confwin, char *label)
     }
 }
 
+static void
+color_selection_bar (void)
+{
+  binding_t quit    = {_("Exit"),     KEY_GENERIC_QUIT};
+  binding_t select  = {_("Select"),   KEY_GENERIC_SELECT};
+  binding_t nocolor = {_("No color"), KEY_GENERIC_CANCEL};
+  binding_t up      = {_("Up"),       KEY_MOVE_UP};
+  binding_t down    = {_("Down"),     KEY_MOVE_DOWN};
+  binding_t left    = {_("Left"),     KEY_MOVE_LEFT};
+  binding_t right   = {_("Right"),    KEY_MOVE_RIGHT};
+
+  
+  binding_t *binding[] = {
+    &quit, &nocolor, &up, &down, &left, &right, &select
+  };
+  int binding_size = sizeof (binding) / sizeof (binding[0]);
+
+  keys_display_bindings_bar (win[STA].p, binding, 0, binding_size);
+}
+
 /* 
  * Used to display available colors in color configuration menu.
  * This is useful for window resizing.
@@ -440,7 +585,6 @@ display_color_config (window_t *cwin, int *mark_fore, int *mark_back,
 #define DEFAULTCOLOR		255
 #define DEFAULTCOLOR_EXT	-1
 #define CURSOR			(32 | A_REVERSE)
-#define SPACE			32
 #define MARK			88
 
   char *fore_txt = _("Foreground");
@@ -448,9 +592,6 @@ display_color_config (window_t *cwin, int *mark_fore, int *mark_back,
   char *default_txt = _("(terminal's default)");
   char *bar = "          ";
   char *box = "[ ]";
-  char *choose_color_1 = _("Use 'X' or SPACE to select a color, "
-			   "'H/L' 'J/K' or arrow keys to move");
-  char *choose_color_2 = _("('0' for no color, 'Q' to exit) :");
   char label[BUFSIZ];
   const unsigned Y = 3;
   const unsigned XOFST = 5;
@@ -555,7 +696,7 @@ display_color_config (window_t *cwin, int *mark_fore, int *mark_back,
     }
 
   mvwaddch (cwin->p, pos[cursor][YPOS], pos[cursor][XPOS] + 1, CURSOR);
-  status_mesg (choose_color_1, choose_color_2);
+  color_selection_bar ();
   wnoutrefresh (win[STA].p);
   wnoutrefresh (cwin->p);
   doupdate ();
@@ -581,7 +722,7 @@ custom_color_config (void)
   display_color_config (&conf_win, &mark_fore, &mark_back, cursor,
 			need_reset, theme_changed);
 
-  while ((ch = wgetch (win[STA].p)) != 'q')
+  while ((ch = keys_getch (win[STA].p)) != KEY_GENERIC_QUIT)
     {
       need_reset = 0;
       theme_changed = 0;
@@ -595,9 +736,7 @@ custom_color_config (void)
 	  need_reset = 1;
 	  break;
 
-	case SPACE:
-	case 'X':
-	case 'x':
+	case KEY_GENERIC_SELECT:
 	  colorize = true;
 	  need_reset = 1;
 	  theme_changed = 1;
@@ -607,35 +746,27 @@ custom_color_config (void)
 	    mark_fore = cursor;
 	  break;
 
-	case 258:
-	case 'J':
-	case 'j':
+	case KEY_MOVE_DOWN:
 	  if (cursor < SIZE - 1)
 	    ++cursor;
 	  break;
 
-	case 259:
-	case 'K':
-	case 'k':
+	case KEY_MOVE_UP:
 	  if (cursor > 0)
 	    --cursor;
 	  break;
 
-	case 260:
-	case 'H':
-	case 'h':
+	case KEY_MOVE_LEFT:
 	  if (cursor > NBUSERCOLORS)
 	    cursor -= (NBUSERCOLORS + 1);
 	  break;
 
-	case 261:
-	case 'L':
-	case 'l':
+	case KEY_MOVE_RIGHT:
 	  if (cursor <= NBUSERCOLORS)
 	    cursor += (NBUSERCOLORS + 1);
 	  break;
 
-	case '0':
+	case KEY_GENERIC_CANCEL:
 	  colorize = false;
 	  need_reset = 1;
 	  break;
@@ -676,8 +807,6 @@ custom_color_theme_name (char *theme_name)
     "cyan",
     "white"
   };
-  const char *error_txt =
-    _("FATAL ERROR in custom_color_theme_name: unknown color\n");
 
   if (!colorize)
     snprintf (theme_name, BUFSIZ, "0");
@@ -692,8 +821,7 @@ custom_color_theme_name (char *theme_name)
 	    color_name[i] = name[color[i]];
 	  else
 	    {
-	      fputs (error_txt, stderr);
-	      exit (EXIT_FAILURE);
+              EXIT (_("unknown color"));
 	      /* NOTREACHED */
 	    }
 	}
@@ -719,38 +847,41 @@ print_general_options (WINDOW *win, conf_t *conf)
   
   y = 0;
   mvwprintw (win, y, XPOS, "[1] %s      ", opt1);
-  print_option_incolor (win, conf->auto_save, y, XPOS + 4 + strlen (opt1));
+  print_bool_option_incolor (win, conf->auto_save, y,
+                             XPOS + 4 + strlen (opt1));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, automatic save is done when quitting)"));
   y += YOFF;
   mvwprintw (win, y, XPOS, "[2] %s      ", opt2);
-  print_option_incolor (win, conf->confirm_quit, y, XPOS + 4 + strlen (opt2));
+  print_bool_option_incolor (win, conf->confirm_quit, y,
+                             XPOS + 4 + strlen (opt2));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, confirmation is required before quitting)"));
   y += YOFF;
   mvwprintw (win, y, XPOS, "[3] %s      ", opt3);
-  print_option_incolor (win, conf->confirm_delete, y, XPOS + 4 + strlen (opt3));
+  print_bool_option_incolor (win, conf->confirm_delete, y,
+                             XPOS + 4 + strlen (opt3));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, confirmation is required "
                "before deleting an event)"));
   y += YOFF;
   mvwprintw (win, y, XPOS, "[4] %s      ", opt4);
-  print_option_incolor (win, conf->skip_system_dialogs, y,
-			XPOS + 4 + strlen (opt4));
+  print_bool_option_incolor (win, conf->skip_system_dialogs, y,
+                             XPOS + 4 + strlen (opt4));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, messages about loaded "
                "and saved data will not be displayed)"));
   y += YOFF;
   mvwprintw (win, y, XPOS, "[5] %s      ", opt5);
-  print_option_incolor (win, conf->skip_progress_bar, y,
-			XPOS + 4 + strlen (opt5));
+  print_bool_option_incolor (win, conf->skip_progress_bar, y,
+                            XPOS + 4 + strlen (opt5));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, progress bar will not be displayed "
                 "when saving data)"));
   y += YOFF;
   mvwprintw (win, y, XPOS, "[6] %s      ", opt6);
-  print_option_incolor (win, calendar_week_begins_on_monday (), y,
-			XPOS + 4 + strlen (opt6));
+  print_bool_option_incolor (win, calendar_week_begins_on_monday (), y,
+                             XPOS + 4 + strlen (opt6));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, monday is the first day of the week, "
                "else it is sunday)"));
@@ -769,11 +900,11 @@ print_general_options (WINDOW *win, conf_t *conf)
   mvwprintw (win, y + 1, XPOS, _("(Format to be used when entering a date: "));
   mvwprintw (win, y + 2, XPOS, _(" 1-mm/dd/yyyy, 2-dd/mm/yyyy, 3-yyyy/mm/dd)"));
 
-  return (y + 3);
+  return y + YOFF;
 }
 
 static void
-general_conf_set_scrsize (scrollwin_t *sw)
+conf_set_scrsize (scrollwin_t *sw)
 {
   sw->win.x = 0;
   sw->win.y = 0;
@@ -792,7 +923,9 @@ custom_general_config (conf_t *conf)
 {
   scrollwin_t cwin;
   char *number_str =
-    _("Enter an option number to change its value [Q to quit] ");
+    _("Enter an option number to change its value");
+  char *keys =
+    _("(Press '^P' or '^N' to move up or down, 'Q' to quit)");
   char *output_datefmt_str =
     _("Enter the date format (see 'man 3 strftime' for possible formats) ");
   char *input_datefmt_str =
@@ -801,11 +934,11 @@ custom_general_config (conf_t *conf)
   char *buf = (char *) malloc (BUFSIZ);
 
   clear ();
-  general_conf_set_scrsize (&cwin);
+  conf_set_scrsize (&cwin);
   snprintf (cwin.label, BUFSIZ, _("CalCurse %s | general options"), VERSION);
   wins_scrollwin_init (&cwin);
   wins_show (cwin.win.p, cwin.label);
-  status_mesg (number_str, "");
+  status_mesg (number_str, keys);
   cwin.total_lines = print_general_options (cwin.pad.p, conf);
   wins_scrollwin_display (&cwin);
 
@@ -818,7 +951,7 @@ custom_general_config (conf_t *conf)
           wins_reset ();
 	  wins_scrollwin_delete (&cwin);
 	  wins_scrollwin_init (&cwin);
-          general_conf_set_scrsize (&cwin);
+          conf_set_scrsize (&cwin);
           wins_show (cwin.win.p, cwin.label);
 	  cwin.first_visible_line = 0;          
 	  delwin (win[STA].p);
@@ -831,11 +964,11 @@ custom_general_config (conf_t *conf)
 	      notify_update_bar ();
 	    }
 	  break;
-        case CTRL ('n'):
-          wins_scrollwin_down (&cwin);
+        case CTRL ('N'):
+          wins_scrollwin_down (&cwin, 1);
 	  break;
-	case CTRL ('p'):
-          wins_scrollwin_up (&cwin);
+        case CTRL ('P'):
+          wins_scrollwin_up (&cwin, 1);
 	  break;
 	case '1':
 	  conf->auto_save = !conf->auto_save;
@@ -863,7 +996,7 @@ custom_general_config (conf_t *conf)
 	    {
 	      strncpy (conf->output_datefmt, buf, strlen (buf) + 1);
 	    }
-	  status_mesg (number_str, "");
+	  status_mesg (number_str, keys);
 	  break;
 	case '8':
 	  status_mesg (input_datefmt_str, "");
@@ -873,13 +1006,225 @@ custom_general_config (conf_t *conf)
 	      if (val >= 1 && val <= 3)
 		conf->input_datefmt = val;
 	    }
-	  status_mesg (number_str, "");
+	  status_mesg (number_str, keys);
 	  break;
 	}
-      status_mesg (number_str, "");
+      status_mesg (number_str, keys);
       cwin.total_lines = print_general_options (cwin.pad.p, conf);
       wins_scrollwin_display (&cwin);
     }
   free (buf);
   wins_scrollwin_delete (&cwin);
 }
+
+
+static void
+print_key_incolor (WINDOW *win, char *option, int pos_y, int pos_x)
+{
+  const int color = ATTR_HIGHEST;
+
+  RETURN_IF (!option, _("Undefined option!"));
+  custom_apply_attr (win, color);
+  mvwprintw (win, pos_y, pos_x, "%s ", option);
+  custom_remove_attr (win, color);
+  wnoutrefresh (win);
+}
+
+static int
+print_keys_bindings (WINDOW *win, int selected_row, int selected_elm, int yoff)
+{
+  const int XPOS = 1;
+  const int EQUALPOS = 23;
+  const int KEYPOS = 25;
+  int noelm, action, y;
+
+  noelm = y = 0;
+  for (action = 0; action < NBKEYS; action++)
+    {
+      char actionstr[BUFSIZ];
+      int nbkeys;
+
+      nbkeys = keys_action_count_keys (action);      
+      snprintf (actionstr, BUFSIZ, "%s", keys_get_label (action));
+      if (action == selected_row)
+        custom_apply_attr (win, ATTR_HIGHEST);
+      mvwprintw (win, y, XPOS, "%s ", actionstr);
+      mvwprintw (win, y, EQUALPOS, "=");
+      if (nbkeys == 0)
+        mvwprintw (win, y, KEYPOS, _("undefined"));        
+      if (action == selected_row)
+        custom_remove_attr (win, ATTR_HIGHEST);
+      if (nbkeys > 0)
+        {
+          if (action == selected_row)
+            {
+              char *key;
+              int pos;
+              
+              pos = KEYPOS;
+              while ((key = keys_action_nkey (action, noelm)) != 0)
+                {
+                  if (noelm == selected_elm)
+                    print_key_incolor (win, key, y, pos);
+                  else
+                    mvwprintw (win, y, pos, "%s ", key);                
+                  noelm++;
+                  pos += strlen (key) + 1;
+                }
+            }
+          else
+            {
+              mvwprintw (win, y, KEYPOS, "%s", keys_action_allkeys (action));
+            }
+        }
+      y += yoff;
+    }
+  
+  return noelm;
+}
+
+static void
+custom_keys_config_bar (void)
+{
+  binding_t quit  = {_("Exit"),     KEY_GENERIC_QUIT};
+  binding_t info  = {_("Key info"), KEY_GENERIC_HELP};
+  binding_t add   = {_("Add key"),  KEY_ADD_ITEM};
+  binding_t del   = {_("Del key"),  KEY_DEL_ITEM};
+  binding_t up    = {_("Up"),       KEY_MOVE_UP};
+  binding_t down  = {_("Down"),     KEY_MOVE_DOWN};
+  binding_t left  = {_("Prev Key"), KEY_MOVE_LEFT};
+  binding_t right = {_("Next Key"), KEY_MOVE_RIGHT};
+    
+  binding_t *binding[] = {
+    &quit, &info, &add, &del, &up, &down, &left, &right
+  };
+  int binding_size = sizeof (binding) / sizeof (binding[0]);
+
+  keys_display_bindings_bar (win[STA].p, binding, 0, binding_size);
+}
+
+void
+custom_keys_config (void)
+{
+  scrollwin_t kwin;
+  int selrow, selelm, firstrow, lastrow, nbrowelm, nbdisplayed;
+  int keyval, used;
+  char *keystr;
+  WINDOW *grabwin;
+  const int LINESPERKEY = 2;
+  const int LABELLINES = 3;
+  
+  clear ();
+  conf_set_scrsize (&kwin);
+  nbdisplayed = (kwin.win.h - LABELLINES) / LINESPERKEY;
+  snprintf (kwin.label, BUFSIZ, _("CalCurse %s | keys configuration"), VERSION);
+  wins_scrollwin_init (&kwin);
+  wins_show (kwin.win.p, kwin.label);
+  custom_keys_config_bar ();
+  selrow = selelm = 0;
+  nbrowelm = print_keys_bindings (kwin.pad.p, selrow, selelm, LINESPERKEY);
+  kwin.total_lines = NBKEYS * LINESPERKEY;
+  wins_scrollwin_display (&kwin);
+  firstrow = 0;
+  lastrow = firstrow + nbdisplayed - 1;
+  for (;;)
+    {
+      int ch;
+
+      ch = keys_getch (win[STA].p);
+      switch (ch)
+	{
+        case KEY_MOVE_UP:
+          if (selrow > 0)
+            {
+              selrow--;
+              selelm = 0;
+              if (selrow == firstrow)
+                {
+                  firstrow--;
+                  lastrow--;
+                  wins_scrollwin_up (&kwin, LINESPERKEY);
+                }
+            }
+	  break;
+	case KEY_MOVE_DOWN:
+          if (selrow < NBKEYS - 1)
+            {
+              selrow++;
+              selelm = 0;
+              if (selrow == lastrow)
+                {
+                  firstrow++;
+                  lastrow++;
+                  wins_scrollwin_down (&kwin, LINESPERKEY);
+                }
+            }
+	  break;
+        case KEY_MOVE_LEFT:
+          if (selelm > 0)
+            selelm--;
+          break;
+        case KEY_MOVE_RIGHT:
+          if (selelm < nbrowelm - 1)
+            selelm++;
+          break;
+        case KEY_GENERIC_HELP:
+          keys_popup_info (selrow);
+          break;
+        case KEY_ADD_ITEM:
+#define WINROW 10
+#define WINCOL 50
+          do
+            {
+              grabwin = popup (WINROW, WINCOL, (row - WINROW) / 2,
+                               (col - WINCOL) / 2,
+                               _("Press the key you want to assign to:"),
+                               keys_get_label (selrow), 0);
+              keyval = wgetch (grabwin);
+              used = keys_assign_binding (keyval, selrow);
+              if (used)
+                {
+                  keys_e action;
+                  
+                  action = keys_get_action (keyval);
+                  ERROR_MSG (
+                             _("This key is already in use for %s, "
+                               "please choose another one."),
+                             keys_get_label (action));
+                  werase (kwin.pad.p);
+                  nbrowelm = print_keys_bindings (kwin.pad.p, selrow, selelm,
+                                                  LINESPERKEY);
+                  wins_scrollwin_display (&kwin);
+                }
+              delwin (grabwin);              
+            }
+          while (used);
+          nbrowelm++;          
+          if (selelm < nbrowelm - 1)
+            selelm++;
+#undef WINROW
+#undef WINCOL
+          break;
+        case KEY_DEL_ITEM:
+          keystr = keys_action_nkey (selrow, selelm);
+          keyval = keys_str2int (keystr);
+          keys_remove_binding (keyval, selrow);
+          nbrowelm--;
+          if (selelm > 0 && selelm <= nbrowelm)
+            selelm--;
+          break;
+        case KEY_GENERIC_QUIT:
+          if (keys_check_missing_bindings () != 0)
+            {
+              ERROR_MSG (_("Some actions do not have any associated "
+                           "key bindings!"));              
+            }
+          wins_scrollwin_delete (&kwin);
+          return;
+	}
+      custom_keys_config_bar ();
+      werase (kwin.pad.p);
+      nbrowelm = print_keys_bindings (kwin.pad.p, selrow, selelm, LINESPERKEY);
+      wins_scrollwin_display (&kwin);
+    }
+}
