@@ -1,4 +1,4 @@
-/*	$calcurse: day.c,v 1.37 2008/04/19 21:05:15 culot Exp $	*/
+/*	$calcurse: day.c,v 1.42 2008/12/15 20:02:00 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -34,6 +34,7 @@
 #include "apoint.h"
 #include "event.h"
 #include "custom.h"
+#include "keys.h"
 #include "day.h"
 
 static struct day_item_s *day_items_ptr;
@@ -453,15 +454,13 @@ day_write_pad (long date, int width, int length, int incolor)
 void
 day_popup_item (void)
 {
-  char *error = _("FATAL ERROR in day_popup_item: unknown item type\n");
-
   if (day_saved_item->type == EVNT || day_saved_item->type == RECUR_EVNT)
     item_in_popup (NULL, NULL, day_saved_item->mesg, _("Event :"));
   else if (day_saved_item->type == APPT || day_saved_item->type == RECUR_APPT)
     item_in_popup (day_saved_item->start, day_saved_item->end,
 		   day_saved_item->mesg, _("Appointment :"));
   else
-    ierror (error, IERROR_FATAL);
+    EXIT (_("unknown item type"));
   /* NOTREACHED */
 }
 
@@ -526,7 +525,7 @@ day_edit_time (long time)
       if (check_time (timestr) != 1 || strlen (timestr) == 0)
 	{
 	  status_mesg (fmt_msg, enter_str);
-	  wgetch (win[STA].p);
+	  (void)wgetch (win[STA].p);
 	}
       else
 	return (timestr);
@@ -558,7 +557,7 @@ update_start_time (long *start, long *dur)
       else
 	{
 	  status_mesg (msg_wrong_time, msg_enter);
-	  wgetch (win[STA].p);
+	  (void)wgetch (win[STA].p);
 	  valid_date = 0;
 	}
     }
@@ -641,7 +640,7 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
 	  if (newfreq == 0)
 	    {
 	      status_mesg (msg_wrong_freq, msg_enter);
-	      wgetch (win[STA].p);
+	      (void)wgetch (win[STA].p);
 	    }
 	}
     }
@@ -684,7 +683,7 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
 	      if (newuntil < start)
 		{
 		  status_mesg (msg_wrong_time, msg_enter);
-		  wgetch (win[STA].p);
+		  (void)wgetch (win[STA].p);
 		  date_entered = 0;
 		}
 	      else
@@ -695,7 +694,7 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
 	      snprintf (outstr, BUFSIZ, msg_fmts,
 			DATEFMT_DESC (conf->input_datefmt));
 	      status_mesg (msg_wrong_date, _(outstr));
-	      wgetch (win[STA].p);
+	      (void)wgetch (win[STA].p);
 	      date_entered = 0;
 	    }
 	}
@@ -729,13 +728,13 @@ day_edit_item (conf_t *conf)
   p = day_get_item (item_num);
   date = calendar_get_slctd_day_sec ();
 
-  ch = 0;
+  ch = -1;
   switch (p->type)
     {
     case RECUR_EVNT:
       re = recur_get_event (date, day_item_nb (date, item_num, RECUR_EVNT));
       status_mesg (_("Edit: (1)Description or (2)Repetition?"), "[1/2] ");
-      while (ch != '1' && ch != '2' && ch != ESCAPE)
+      while (ch != '1' && ch != '2' && ch != KEY_GENERIC_CANCEL)
 	ch = wgetch (win[STA].p);
       switch (ch)
 	{
@@ -758,7 +757,7 @@ day_edit_item (conf_t *conf)
       status_mesg (_("Edit: (1)Start time, (2)End time, "
 		     "(3)Description or (4)Repetition?"), "[1/2/3/4] ");
       while (ch != STRT && ch != END && ch != DESC &&
-	     ch != REPT && ch != ESCAPE)
+	     ch != REPT && ch != KEY_GENERIC_CANCEL)
 	ch = wgetch (win[STA].p);
       switch (ch)
 	{
@@ -774,7 +773,7 @@ day_edit_item (conf_t *conf)
 	case REPT:
 	  update_rept (&ra->rpt, ra->start, conf);
 	  break;
-	case ESCAPE:
+	case KEY_GENERIC_CANCEL:
 	  return;
 	}
       break;
@@ -782,7 +781,7 @@ day_edit_item (conf_t *conf)
       a = apoint_get (date, day_item_nb (date, item_num, APPT));
       status_mesg (_("Edit: (1)Start time, (2)End time "
 		     "or (3)Description?"), "[1/2/3] ");
-      while (ch != STRT && ch != END && ch != DESC && ch != ESCAPE)
+      while (ch != STRT && ch != END && ch != DESC && ch != KEY_GENERIC_CANCEL)
 	ch = wgetch (win[STA].p);
       switch (ch)
 	{
@@ -795,7 +794,7 @@ day_edit_item (conf_t *conf)
 	case DESC:
 	  update_desc (&a->mesg);
 	  break;
-	case ESCAPE:
+	case KEY_GENERIC_CANCEL:
 	  return;
 	}
       break;
@@ -820,9 +819,10 @@ day_erase_item (long date, int item_number, erase_flag_e flag)
 	"Delete (i)tem or just its (n)ote ?");
   char *note_choice = _("[i/n] ");
   char *erase_choice = _("[a/o] ");
-  int ch = 0, ans;
+  int ch, ans;
   unsigned delete_whole;
 
+  ch = -1;
   p = day_get_item (item_number);
   if (flag == ERASE_DONT_FORCE)
     {
@@ -851,7 +851,7 @@ day_erase_item (long date, int item_number, erase_flag_e flag)
     {
       if (flag == ERASE_FORCE_ONLY_NOTE)
 	ch = 'a';
-      while ((ch != 'a') && (ch != 'o') && (ch != ESCAPE))
+      while ((ch != 'a') && (ch != 'o') && (ch != KEY_GENERIC_CANCEL))
 	{
 	  status_mesg (erase_warning, erase_choice);
 	  ch = wgetch (win[STA].p);
@@ -878,7 +878,10 @@ day_erase_item (long date, int item_number, erase_flag_e flag)
 	  recur_apoint_erase (date, p->appt_pos, delete_whole, flag);
 	}
     }
-  return (p->type);
+  if (flag == ERASE_FORCE_ONLY_NOTE)
+    return 0;
+  else
+    return (p->type);
 }
 
 /* Returns a structure containing the selected item. */
