@@ -1,4 +1,4 @@
-/*	$calcurse: event.h,v 1.5 2008/01/20 10:45:38 culot Exp $	*/
+/*	$calcurse: event.h,v 1.6 2008/04/12 21:14:03 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -32,21 +32,22 @@
 #define HRMIN_SIZE 6
 #define MESG_MAXSIZE 256
 
-struct event_s {
-	struct event_s *next;
-	int id;	        /* event identifier */
-        long day;	/* seconds since 1 jan 1970 */
-	char *mesg;
-	char *note;
+struct event_s
+{
+  struct event_s *next;
+  int             id;		/* event identifier */
+  long            day;		/* seconds since 1 jan 1970 */
+  char           *mesg;
+  char           *note;
 };
 
 extern struct event_s *eventlist;
 
-struct event_s *event_new(char *, char *, long, int);
-unsigned 	event_inday(struct event_s *, long);
-void 		event_write(struct event_s *, FILE *);
-struct event_s *event_scan(FILE *, struct tm, int, char *);
-struct event_s *event_get(long, int);
-void 		event_delete_bynum(long, unsigned, erase_flag_e);
+struct event_s *event_new (char *, char *, long, int);
+unsigned        event_inday (struct event_s *, long);
+void            event_write (struct event_s *, FILE *);
+struct event_s *event_scan (FILE *, struct tm, int, char *);
+struct event_s *event_get (long, int);
+void            event_delete_bynum (long, unsigned, erase_flag_e);
 
 #endif /* CALCURSE_EVENT_H */
