@@ -1,4 +1,4 @@
-/*	$calcurse: io.c,v 1.40 2008/09/29 06:27:53 culot Exp $	*/
+/*	$calcurse: io.c,v 1.49 2008/12/14 15:54:51 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -39,24 +39,31 @@
 #include "event.h"
 #include "apoint.h"
 #include "recur.h"
+#include "keys.h"
+#include "htable.h"
 #include "io.h"
 
 #define ICALDATEFMT      "%Y%m%d"
 #define ICALDATETIMEFMT  "%Y%m%dT%H%M%S"
 
-#define STRING_BUILD(str) {str, sizeof (str) - 1}
-
-typedef enum
-{
+typedef enum {
   PROGRESS_BAR_SAVE,
   PROGRESS_BAR_LOAD,
   PROGRESS_BAR_EXPORT
 } progress_bar_t;
 
-typedef struct {
-  const char *str;
-  const int len;
-} string_t;
+enum {
+  PROGRESS_BAR_CONF,
+  PROGRESS_BAR_TODO,
+  PROGRESS_BAR_APTS,
+  PROGRESS_BAR_KEYS
+};
+
+enum {
+  PROGRESS_BAR_EXPORT_EVENTS,
+  PROGRESS_BAR_EXPORT_APOINTS,
+  PROGRESS_BAR_EXPORT_TODO
+};
 
 typedef enum {
   ICAL_VEVENT,
@@ -77,6 +84,12 @@ typedef struct {
   unsigned      count;
 } ical_rpt_t;
 
+struct ht_keybindings_s {
+  char *label;
+  keys_e key;
+  HTABLE_ENTRY (ht_keybindings_s);
+};
+
 /* Type definition for callbacks to multiple-mode export functions. */
 typedef void (*cb_export_t)(FILE *);
 typedef void (*cb_dump_t)(FILE *, long, long, char *);
@@ -121,50 +134,56 @@ static void
 progress_bar (progress_bar_t type, int progress)
 {
 #define SLEEPTIME	125000
-#define STEPS		3
-#define LABELENGTH	15
-
-  int i, step;
+#define NBFILES		4
+#define NBEXPORTED      3
+#define LABELENGTH      15
+  int i, step, steps;
   char *mesg_sav = _("Saving...");
   char *mesg_load = _("Loading...");
   char *mesg_export = _("Exporting...");
+  char *error_msg = _("Internal error while displaying progress bar");
   char *barchar = "|";
-  char file[STEPS][LABELENGTH] = {
+  char *file[NBFILES] = {
     "[    conf    ]",
     "[    todo    ]",
-    "[    apts    ]"
+    "[    apts    ]",
+    "[    keys    ]"
   };
-  char data[STEPS][LABELENGTH] = {
+  char *data[NBEXPORTED] = {
     "[   events   ]",
     "[appointments]",
     "[    todo    ]"
   };
-  int ipos = LABELENGTH + 2;
-  int epos[STEPS];
+  int ipos =  LABELENGTH + 2;
+  int epos[NBFILES];
 
   /* progress bar length init. */
   ipos = LABELENGTH + 2;
-  step = floor (col / (STEPS + 1));
-  for (i = 0; i < STEPS - 1; i++)
+  steps = (type == PROGRESS_BAR_EXPORT) ? NBEXPORTED : NBFILES;
+  step = floor (col / (steps + 1));
+  for (i = 0; i < steps - 1; i++)
     epos[i] = (i + 2) * step;
-  epos[STEPS - 1] = col - 2;
+  epos[steps - 1] = col - 2;
 
   switch (type)
     {
     case PROGRESS_BAR_SAVE:
+      EXIT_IF (progress < 0 || progress > PROGRESS_BAR_KEYS, error_msg);
       status_mesg (mesg_sav, file[progress]);
       break;
     case PROGRESS_BAR_LOAD:
+      EXIT_IF (progress < 0 || progress > PROGRESS_BAR_KEYS, error_msg);
       status_mesg (mesg_load, file[progress]);
       break;
     case PROGRESS_BAR_EXPORT:
+      EXIT_IF (progress < 0 || progress > PROGRESS_BAR_EXPORT_TODO, error_msg);
       status_mesg (mesg_export, data[progress]);
       break;
     }
 
   /* Draw the progress bar. */
   mvwprintw (win[STA].p, 1, ipos, barchar);
-  mvwprintw (win[STA].p, 1, epos[STEPS - 1], barchar);
+  mvwprintw (win[STA].p, 1, epos[steps - 1], barchar);
   custom_apply_attr (win[STA].p, ATTR_HIGHEST);
   for (i = ipos + 1; i < epos[progress]; i++)
     mvwaddch (win[STA].p, 1, i, ' ' | A_REVERSE);
@@ -172,6 +191,10 @@ progress_bar (progress_bar_t type, int progress)
   wmove (win[STA].p, 0, 0);
   wrefresh (win[STA].p);
   usleep (SLEEPTIME);
+#undef SLEEPTIME
+#undef NBFILES
+#undef NBEXPORTED
+#undef LABELENGTH
 }
 
 /* Ask user for a file name to export data to. */
@@ -207,7 +230,7 @@ get_export_stream (export_type_t type)
       if (stream == NULL)
 	{
 	  status_mesg (wrong_name, press_enter);
-	  wgetch (win[STA].p);
+	  (void)wgetch (win[STA].p);
 	}
     }
   free (stream_name);
@@ -215,7 +238,8 @@ get_export_stream (export_type_t type)
   return (stream);
 }
 
-/* Travel through each occurence of an item, and execute the given callback
+/*
+ * Travel through each occurence of an item, and execute the given callback
  * (mainly used to export data).
  */
 static void
@@ -223,14 +247,23 @@ foreach_date_dump (const long date_end, struct rpt_s *rpt, struct days_s *exc,
                    long item_first_date, long item_dur, char *item_mesg,
                    cb_dump_t cb_dump, FILE *stream)
 {
-  long date;
+  long date, item_time;
+  struct tm lt;
+  time_t t;
+
+  t = item_first_date;
+  lt = *localtime (&t);
+  lt.tm_hour = lt.tm_min = lt.tm_sec = 0;
+  lt.tm_isdst = -1;
+  date = mktime (&lt);
+  item_time = item_first_date - date;
 
-  date = item_first_date;
   while (date <= date_end && date <= rpt->until)
     {
-      if (!recur_day_is_exc (date, exc))
+      if (recur_item_inday (item_first_date, exc, rpt->type, rpt->freq,
+                            rpt->until, date))
         {
-          (*cb_dump)(stream, date, item_dur, item_mesg);
+          (*cb_dump)(stream, date + item_time, item_dur, item_mesg);
         }
       switch (rpt->type)
         {
@@ -247,9 +280,7 @@ foreach_date_dump (const long date_end, struct rpt_s *rpt, struct days_s *exc,
           date = date_sec_change (date, rpt->freq * 12, 0);
           break;
         default:
-          fputs (_("FATAL ERROR in foreach_date_dump: "
-                   "incoherent repetition type\n"), stderr);
-          exit (EXIT_FAILURE);
+          EXIT (_("incoherent repetition type"));
           /* NOTREACHED */
           break;
         }
@@ -406,10 +437,7 @@ pcal_export_recur_events (FILE *stream)
               fprintf (stream, "%s  %s\n", pcal_date, i->mesg);
               break;
             default:
-              fputs (_("FATAL ERROR in pcal_export_recur_events: "
-                       "incoherent repetition type\n"), stderr);
-              exit (EXIT_FAILURE);
-              break;
+              EXIT (_("incoherent repetition type"));
             }
         }
       else
@@ -542,10 +570,7 @@ pcal_export_recur_apoints (FILE *stream)
                        pcal_beg, pcal_end, i->mesg);
               break;
             default:
-              fputs (_("FATAL ERROR in pcal_export_recur_apoints: "
-                       "incoherent repetition type\n"), stderr);
-              exit (EXIT_FAILURE);
-              break;
+              EXIT (_("incoherent repetition type"));
             }
         }
       else
@@ -647,6 +672,7 @@ io_init (char *cfile, char *datadir)
       snprintf (path_conf, BUFSIZ, "%s/" CONF_PATH_NAME, home);
       snprintf (path_notes, BUFSIZ, "%s/" NOTES_DIR_NAME, home);
       snprintf (path_apts, BUFSIZ, "%s/" APTS_PATH_NAME, home);
+      snprintf (path_keys, BUFSIZ, "%s/" KEYS_PATH_NAME, home);      
     }
   else
     {
@@ -658,6 +684,7 @@ io_init (char *cfile, char *datadir)
       snprintf (path_dir, BUFSIZ, "%s/" DIR_NAME, home);
       snprintf (path_todo, BUFSIZ, "%s/" TODO_PATH, home);
       snprintf (path_conf, BUFSIZ, "%s/" CONF_PATH, home);
+      snprintf (path_keys, BUFSIZ, "%s/" KEYS_PATH, home);      
       snprintf (path_notes, BUFSIZ, "%s/" NOTES_DIR, home);
       if (cfile == NULL)
         {
@@ -723,7 +750,7 @@ io_extract_data (char *dst_data, const char *org, int len)
 
 /* Save the calendar data */
 void
-io_save_cal (io_mode_t mode, conf_t *conf)
+io_save_cal (conf_t *conf)
 {
   FILE *data_file;
   struct event_s *k;
@@ -745,13 +772,12 @@ io_save_cal (io_mode_t mode, conf_t *conf)
   char *enter = _("Press [ENTER] to continue");
   bool show_bar = false;
 
-  if (mode == IO_MODE_INTERACTIVE && !conf->skip_progress_bar)
+  if (ui_mode == UI_CURSES && !conf->skip_progress_bar)
     show_bar = true;
 
   /* Save the user configuration. */
-
   if (show_bar)
-    progress_bar (PROGRESS_BAR_SAVE, 0);
+    progress_bar (PROGRESS_BAR_SAVE, PROGRESS_BAR_CONF);
   data_file = fopen (path_conf, "w");
   if (data_file == NULL)
     ERROR_MSG (access_pb);
@@ -806,7 +832,7 @@ io_save_cal (io_mode_t mode, conf_t *conf)
       fprintf (data_file, "layout=\n");
       fprintf (data_file, "%d\n", wins_layout ());
 
-      if (mode == IO_MODE_INTERACTIVE)
+      if (ui_mode == UI_CURSES)
         pthread_mutex_lock (&nbar->mutex);
       fprintf (data_file,
 	       "\n# If this option is set to yes, "
@@ -848,7 +874,7 @@ io_save_cal (io_mode_t mode, conf_t *conf)
       fprintf (data_file, "input_datefmt=\n");
       fprintf (data_file, "%d\n", conf->input_datefmt);
 
-      if (mode == IO_MODE_INTERACTIVE)
+      if (ui_mode == UI_CURSES)
         pthread_mutex_unlock (&nbar->mutex);
 
       fclose (data_file);
@@ -856,7 +882,7 @@ io_save_cal (io_mode_t mode, conf_t *conf)
 
   /* Save the todo data file. */
   if (show_bar)
-    progress_bar (PROGRESS_BAR_SAVE, 1);
+    progress_bar (PROGRESS_BAR_SAVE, PROGRESS_BAR_TODO);
   data_file = fopen (path_todo, "w");
   if (data_file == NULL)
     ERROR_MSG (access_pb);
@@ -878,7 +904,7 @@ io_save_cal (io_mode_t mode, conf_t *conf)
    * Recursive items are written first.
    */
   if (show_bar)
-    progress_bar (PROGRESS_BAR_SAVE, 2);
+    progress_bar (PROGRESS_BAR_SAVE, PROGRESS_BAR_APTS);
   data_file = fopen (path_apts, "w");
   if (data_file == NULL)
     ERROR_MSG (access_pb);
@@ -886,11 +912,11 @@ io_save_cal (io_mode_t mode, conf_t *conf)
     {
       recur_save_data (data_file);
 
-      if (mode == IO_MODE_INTERACTIVE)
+      if (ui_mode == UI_CURSES)
         pthread_mutex_lock (&(alist_p->mutex));
       for (j = alist_p->root; j != 0; j = j->next)
 	apoint_write (j, data_file);
-      if (mode == IO_MODE_INTERACTIVE)
+      if (ui_mode == UI_CURSES)
         pthread_mutex_unlock (&(alist_p->mutex));
 
       for (k = eventlist; k != 0; k = k->next)
@@ -898,11 +924,23 @@ io_save_cal (io_mode_t mode, conf_t *conf)
       fclose (data_file);
     }
 
+  /* Save user-defined keys */
+  if (show_bar)
+    progress_bar (PROGRESS_BAR_SAVE, PROGRESS_BAR_KEYS);
+  data_file = fopen (path_keys, "w");
+  if (data_file == NULL)
+    ERROR_MSG (access_pb);
+  else
+    {
+      keys_save_bindings (data_file);
+      fclose (data_file);
+    }
+  
   /* Print a message telling data were saved */
-  if (mode == IO_MODE_INTERACTIVE && !conf->skip_system_dialogs)
+  if (ui_mode == UI_CURSES && !conf->skip_system_dialogs)
     {
       status_mesg (save_success, enter);
-      wgetch (win[STA].p);
+      (void)wgetch (win[STA].p);
     }
 }
 
@@ -923,8 +961,6 @@ io_load_app (void)
   int freq;
   char type, state = 0L;
   char note[NOTESIZ + 1], *notep;
-  char *error =
-    _("FATAL ERROR in io_load_app: wrong format in the appointment or event\n");
 
   t = time (NULL);
   lt = localtime (&t);
@@ -945,9 +981,7 @@ io_load_app (void)
       if (fscanf (data_file, "%u / %u / %u ",
 		  &start.tm_mon, &start.tm_mday, &start.tm_year) != 3)
 	{
-	  fputs (_("FATAL ERROR in io_load_app: "
-		   "syntax error in the item date\n"), stderr);
-	  exit (EXIT_FAILURE);
+          EXIT (_("syntax error in the item date"));
 	}
 
       /* Read the next character : if it is an '@' then we have
@@ -961,9 +995,7 @@ io_load_app (void)
 	is_event = 1;
       else
 	{
-	  fputs (_("FATAL ERROR in io_load_app: "
-		   "no event nor appointment found\n"), stderr);
-	  exit (EXIT_FAILURE);
+          EXIT (_("no event nor appointment found"));
 	}
       ungetc (c, data_file);
 
@@ -980,9 +1012,9 @@ io_load_app (void)
 	  fscanf (data_file, "[%d] ", &id);
 	}
       else
-	{			/* NOT REACHED */
-	  fputs (error, stderr);
-	  exit (EXIT_FAILURE);
+	{
+          EXIT (_("wrong format in the appointment or event"));
+          /* NOTREACHED */
 	}
 
       /* Check if we have a recursive item. */
@@ -1027,9 +1059,9 @@ io_load_app (void)
 	      until.tm_year = 0;
 	    }
 	  else
-	    {			/* NOT REACHED */
-	      fputs (error, stderr);
-	      exit (EXIT_FAILURE);
+	    {
+              EXIT (_("wrong format in the appointment or event"));
+              /* NOTREACHED */
 	    }
 	}
       else
@@ -1092,9 +1124,9 @@ io_load_app (void)
 	    }
 	}
       else
-	{			/* NOT REACHED */
-	  fputs (error, stderr);
-	  exit (EXIT_FAILURE);
+	{
+          EXIT (_("wrong format in the appointment or event"));
+          /* NOTREACHED */
 	}
     }
   fclose (data_file);
@@ -1116,7 +1148,7 @@ io_load_todo (void)
   if (data_file == NULL)
     {
       status_mesg (mesg_line1, mesg_line2);
-      wgetch (win[STA].p);
+      (void)wgetch (win[STA].p);
     }
   for (;;)
     {
@@ -1159,6 +1191,176 @@ io_load_todo (void)
   todo_set_nb (nb_tod);
 }
 
+static void
+load_keys_ht_getkey (struct ht_keybindings_s *data, char **key, int *len)
+{
+  *key = data->label;
+  *len = strlen (data->label);
+}
+
+static int
+load_keys_ht_compare (struct ht_keybindings_s *data1,
+                      struct ht_keybindings_s *data2)
+{
+  const int KEYLEN = strlen (data1->label);
+
+  if (strlen (data2->label) == KEYLEN
+      && !memcmp (data1->label, data2->label, KEYLEN))
+    return 0;
+  else
+    return 1;
+}
+
+/*
+ * isblank(3) is protected by the __BSD_VISIBLE macro and this fails to be
+ * visible in some specific cases. Thus replace it by the following is_blank()
+ * function.
+ */
+static int is_blank (int c)
+{
+  return c == ' ' || c == '\t';
+}
+
+/*
+ * Load user-definable keys from file.
+ * A hash table is used to speed up loading process in avoiding string
+ * comparisons.
+ * A log file is also built in case some errors were found in the key
+ * configuration file.
+ */
+void
+io_load_keys (char *pager)
+{
+  struct ht_keybindings_s keys[NBKEYS];
+  FILE *keyfp;
+  char buf[BUFSIZ];
+  io_file_t *log;
+  int i, skipped, loaded, line;
+  const int MAX_ERRORS = 5;
+  
+#define HSIZE 256
+  HTABLE_HEAD (ht_keybindings, HSIZE, ht_keybindings_s) ht_keys =
+    HTABLE_INITIALIZER (&ht_keys);
+  
+  HTABLE_GENERATE (ht_keybindings, ht_keybindings_s, load_keys_ht_getkey,
+                   load_keys_ht_compare);
+
+  for (i = 0; i < NBKEYS; i++)
+    {
+      keys[i].key = (keys_e)i;
+      keys[i].label = keys_get_label ((keys_e)i);
+      HTABLE_INSERT (ht_keybindings, &ht_keys, &keys[i]);
+    }
+
+  keyfp = fopen (path_keys, "r");
+  EXIT_IF (keyfp == NULL, _("could not find any key file."));
+  log = io_log_init ();
+  skipped = loaded = line = 0;
+  while (fgets (buf, BUFSIZ, keyfp) != NULL)
+    {
+      char key_label[BUFSIZ], *p;
+      struct ht_keybindings_s *ht_elm, ht_entry;
+      const int AWAITED = 1;
+      int assigned;
+
+      line++;
+      if (skipped > MAX_ERRORS)
+        {
+          char *too_many =
+            _("\nToo many errors while reading configuration file!\n"
+              "Please backup your keys file, remove it from directory, "
+              "and launch calcurse again.\n");
+
+          io_log_print (log, line, too_many);
+          break;
+        }
+      for (p = buf; is_blank ((int)*p); p++)
+        ;
+      if (p != buf)
+        memmove (buf, p, strlen (p));
+      if (buf[0] == '#' || buf[0] == '\n')
+        continue;
+      
+      if (sscanf (buf, "%s", key_label) != AWAITED)
+        {
+          skipped++;
+          io_log_print (log, line, _("Could not read key label"));
+          continue;
+        }
+      ht_entry.label = key_label;
+      p = buf + strlen (key_label) + 1;      
+      ht_elm = HTABLE_LOOKUP (ht_keybindings, &ht_keys, &ht_entry);
+      if (!ht_elm)
+        {
+          skipped++;
+          io_log_print (log, line, _("Key label not recognized"));
+          continue;
+        }
+      assigned = 0;
+      for (;;)
+        {
+          char key_ch[BUFSIZ], tmpbuf[BUFSIZ];
+
+          while (*p == ' ')
+            p++;
+          strncpy (tmpbuf, p, BUFSIZ);
+          if (sscanf (tmpbuf, "%s", key_ch) == AWAITED)
+            {
+              int ch;
+                    
+              if ((ch = keys_str2int (key_ch)) < 0)
+                {
+                  char unknown_key[BUFSIZ];
+
+                  skipped++;
+                  snprintf (unknown_key, BUFSIZ, _("Error reading key: \"%s\""),
+                            key_ch);
+                  io_log_print (log, line, unknown_key);
+                }
+              else
+                {
+                  int used;
+                  
+                  used = keys_assign_binding (ch, ht_elm->key);
+                  if (used)
+                    {
+                      char already_assigned[BUFSIZ];
+
+                      skipped++;
+                      snprintf (already_assigned, BUFSIZ,
+                                _("\"%s\" assigned multiple times!"), key_ch);
+                      io_log_print (log, line, already_assigned);
+                    }
+                  else
+                    assigned++;
+                }
+              p += strlen (key_ch) + 1;
+            }
+          else
+            {
+              if (assigned)
+                loaded++;
+              break;
+            }
+        }
+    }
+  fclose (keyfp);
+  fclose (log->fd);
+  if (skipped > 0)
+    {
+      char *view_log =
+        _("There were some errors when loading keys file, see log file ?");
+      
+      io_log_display (log, view_log, pager);
+    }
+  io_log_free (log);
+  EXIT_IF (skipped > MAX_ERRORS,
+           _("Too many errors while reading keys file, aborting..."));
+  if (loaded < NBKEYS)
+    ERROR_MSG (_("Some actions do not have any associated key bindings!"));
+#undef HSIZE
+}
+
 static void
 check_directory (char *dir, int *missing)
 {
@@ -1203,22 +1405,30 @@ check_file (char *file, int *missing)
  *                 |
  *                 +--- notes/
  *                 |___ conf
+ *                 |___ keys
  *                 |___ apts
  *                 |___ todo
  */
 int
 io_check_data_files (void)
 {
-  int missing;
+  int missing, missing_keys;
 
-  missing = 0;
+  missing = missing_keys = 0;
   errno = 0;
   check_directory (path_dir, &missing);
   check_directory (path_notes, &missing);
   check_file (path_todo, &missing);
   check_file (path_apts, &missing);
   check_file (path_conf, &missing);
-  return (missing);
+  check_file (path_keys, &missing_keys);
+  if (missing_keys)
+    {
+      missing++;
+      keys_dump_defaults (path_keys);
+    }
+
+  return missing;
 }
 
 /* Draw the startup screen */
@@ -1233,61 +1443,57 @@ io_startup_screen (bool skip_dialogs, int no_data_file)
   if (no_data_file != 0)
     {
       status_mesg (welcome_mesg, enter);
-      wgetch (win[STA].p);
+      (void)wgetch (win[STA].p);
     }
   else if (!skip_dialogs)
     {
       status_mesg (data_mesg, enter);
-      wgetch (win[STA].p);
+      (void)wgetch (win[STA].p);
     }
 }
 
 /* Export calcurse data. */
 void
-io_export_data (io_mode_t mode, export_type_t type, conf_t *conf)
+io_export_data (export_type_t type, conf_t *conf)
 {
   FILE *stream;
-  char *wrong_mode = _("FATAL ERROR in io_export_data: wrong export mode\n");
-  char *wrong_type = _("FATAL ERROR in io_export_data: unknown export type\n");
   char *success = _("The data were successfully exported");
   char *enter = _("Press [ENTER] to continue");
 
   if (type < IO_EXPORT_ICAL || type >= IO_EXPORT_NBTYPES)
+    EXIT (_("unknown export type"));
+
+  stream = 0;
+  switch (ui_mode)
     {
-      fputs (wrong_type, stderr);
-      exit (EXIT_FAILURE);
-    }
-  switch (mode)
-    {
-    case IO_MODE_NONINTERACTIVE:
+    case UI_CMDLINE:
       stream = stdout;
       break;
-    case IO_MODE_INTERACTIVE:
+    case UI_CURSES:
       stream = get_export_stream (type);
       break;
     default:
-      fputs (wrong_mode, stderr);
-      exit (EXIT_FAILURE);
+      EXIT (_("wrong export mode"));
       /* NOTREACHED */
     }
 
-  if (stream == NULL)
+  if (stream == 0)
     return;
 
   cb_export_header[type] (stream);
 
-  if (!conf->skip_progress_bar && mode == IO_MODE_INTERACTIVE)
-    progress_bar (PROGRESS_BAR_EXPORT, 0);
+  if (!conf->skip_progress_bar && ui_mode == UI_CURSES)
+    progress_bar (PROGRESS_BAR_EXPORT, PROGRESS_BAR_EXPORT_EVENTS);
   cb_export_recur_events[type] (stream);
   cb_export_events[type] (stream);
 
-  if (!conf->skip_progress_bar && mode == IO_MODE_INTERACTIVE)
-    progress_bar (PROGRESS_BAR_EXPORT, 1);
+  if (!conf->skip_progress_bar && ui_mode == UI_CURSES)
+    progress_bar (PROGRESS_BAR_EXPORT, PROGRESS_BAR_EXPORT_APOINTS);
   cb_export_recur_apoints[type] (stream);
   cb_export_apoints[type] (stream);
 
-  if (!conf->skip_progress_bar && mode == IO_MODE_INTERACTIVE)
-    progress_bar (PROGRESS_BAR_EXPORT, 2);
+  if (!conf->skip_progress_bar && ui_mode == UI_CURSES)
+    progress_bar (PROGRESS_BAR_EXPORT, PROGRESS_BAR_EXPORT_TODO);
   cb_export_todo[type] (stream);
 
   cb_export_footer[type] (stream);
@@ -1295,10 +1501,10 @@ io_export_data (io_mode_t mode, export_type_t type, conf_t *conf)
   if (stream != stdout)
     fclose (stream);
 
-  if (!conf->skip_system_dialogs && mode == IO_MODE_INTERACTIVE)
+  if (!conf->skip_system_dialogs && ui_mode == UI_CURSES)
     {
       status_mesg (success, enter);
-      wgetch (win[STA].p);
+      (void)wgetch (win[STA].p);
     }
 }
 
@@ -1345,8 +1551,9 @@ ical_log_init (FILE *log, float version)
     "|  * LINE is the line in the input stream at which this item begins |\n"
     "|  * DESCRIPTION indicates why the item could not be imported       |\n"
     "+-------------------------------------------------------------------+\n\n";
-    
-  fprintf (log, header, version);
+
+  if (log)
+    fprintf (log, header, version);
 }
 
 /*
@@ -1360,9 +1567,9 @@ ical_log (FILE *log, ical_types_e type, unsigned lineno, char *msg)
 {
   const char *typestr[ICAL_TYPES] = {"VEVENT", "VTODO"};
 
-  RETURN_IF (type < 0 || type >= ICAL_TYPES,
-             _("ERROR in ical_log: unknown ical type"));
-  fprintf (log, "%s [%d]: %s\n", typestr[type], lineno, msg);
+  RETURN_IF (type < 0 || type >= ICAL_TYPES, _("unknown ical type"));
+  if (log)
+    fprintf (log, "%s [%d]: %s\n", typestr[type], lineno, msg);
 }
 
 static void
@@ -1490,7 +1697,6 @@ ical_unformat_line (char *line)
 static char *
 ical_unfold_content (FILE *fd, char *line, unsigned *lineno)
 {
-  const int CHAR_SPACE = 32, CHAR_TAB = 9;
   char *content;
   int c;
 
@@ -1501,7 +1707,7 @@ ical_unfold_content (FILE *fd, char *line, unsigned *lineno)
   for (;;)
     {
       c = getc (fd);
-      if (c == CHAR_SPACE || c == CHAR_TAB)
+      if (c == SPACE || c == TAB)
         {
           char buf[BUFSIZ];
           
@@ -2338,7 +2544,7 @@ get_import_stream (export_type_t type)
       if (stream == NULL)
 	{
 	  status_mesg (wrong_file, press_enter);
-	  wgetch (win[STA].p);
+	  (void)wgetch (win[STA].p);
 	}
     }
   mem_free (stream_name);
@@ -2353,10 +2559,8 @@ get_import_stream (export_type_t type)
  * and is cleared at the end.
  */
 void
-io_import_data (io_mode_t mode, import_type_t type, conf_t *conf,
-                char *stream_name)
+io_import_data (import_type_t type, conf_t *conf, char *stream_name)
 {
-  const char *logprefix = "/tmp/calcurse_log.";
   const string_t vevent = STRING_BUILD ("BEGIN:VEVENT");
   const string_t vtodo = STRING_BUILD ("BEGIN:VTODO");
   char *proc_report = _("Import process report: %04d lines read ");
@@ -2364,28 +2568,28 @@ io_import_data (io_mode_t mode, import_type_t type, conf_t *conf,
     _("%d apps / %d events / %d todos / %d skipped ");
   char *lines_stats_interactive =
     _("%d apps / %d events / %d todos / %d skipped ([ENTER] to continue)");
-  char *logname, flogname[BUFSIZ], buf[BUFSIZ];
-  FILE *stream = NULL, *logfd;
+  char buf[BUFSIZ];
+  FILE *stream = NULL;
+  io_file_t *log;
   float ical_version;
   struct {
     unsigned events, apoints, todos, lines, skipped;
   } stats;
 
-  EXIT_IF (type < 0 || type >= IO_IMPORT_NBTYPES,
-           _("FATAL ERROR in io_import_data: unknown import type"));
-  switch (mode)
+  EXIT_IF (type < 0 || type >= IO_IMPORT_NBTYPES, _("unknown import type"));
+  switch (ui_mode)
     {
-    case IO_MODE_NONINTERACTIVE:
+    case UI_CMDLINE:
       stream = fopen (stream_name, "r");
       EXIT_IF (stream == NULL,
                _("FATAL ERROR: the input file cannot be accessed, "
                  "Aborting..."));
       break;
-    case IO_MODE_INTERACTIVE:
+    case UI_CURSES:
       stream = get_import_stream (type);
       break;
     default:
-      EXIT (_("FATAL ERROR in io_import_data: wrong import mode"));
+      EXIT (_("FATAL ERROR: wrong import mode"));
       /* NOTREACHED */
     }
 
@@ -2398,15 +2602,14 @@ io_import_data (io_mode_t mode, import_type_t type, conf_t *conf,
              _("Warning: ical header malformed or wrong version number. "
                "Aborting..."));
 
-  logname = new_tempfile (logprefix, NOTESIZ);
-  RETURN_IF (logname == NULL,
-             _("Warning: could not create new note file to store "
-               "description. Aborting...\n"));
-  snprintf (flogname, BUFSIZ, "%s%s", logprefix, logname);
-  logfd = fopen (flogname, "w");
-  RETURN_IF (logfd == NULL, 
-             _("Warning: could not open temporary log file, Aborting..."));
-  ical_log_init (logfd, ical_version);
+  log = io_log_init ();
+  if (log == 0)
+    {
+      if (stream != stdin)
+        fclose (stream);
+      return;
+    }
+  ical_log_init (log->fd, ical_version);
 
   while (fgets (buf, BUFSIZ, stream) != NULL)
     {
@@ -2414,19 +2617,19 @@ io_import_data (io_mode_t mode, import_type_t type, conf_t *conf,
       str_toupper (buf);
       if (strncmp (buf, vevent.str, vevent.len) == 0)
         {
-          ical_read_event (stream, logfd, &stats.events, &stats.apoints,
+          ical_read_event (stream, log->fd, &stats.events, &stats.apoints,
                            &stats.skipped, &stats.lines);
         }
       else if (strncmp (buf, vtodo.str, vtodo.len) == 0)
         {
-          ical_read_todo (stream, logfd, &stats.todos, &stats.skipped,
+          ical_read_todo (stream, log->fd, &stats.todos, &stats.skipped,
                           &stats.lines);
         }
     }
   if (stream != stdin)
     fclose (stream);
 
-  if (mode == IO_MODE_INTERACTIVE && !conf->skip_system_dialogs)
+  if (ui_mode == UI_CURSES && !conf->skip_system_dialogs)
     {
       char read[BUFSIZ], stat[BUFSIZ];
 
@@ -2434,9 +2637,9 @@ io_import_data (io_mode_t mode, import_type_t type, conf_t *conf,
       snprintf (stat, BUFSIZ, lines_stats_interactive, stats.apoints,
                 stats.events, stats.todos, stats.skipped);
       status_mesg (read, stat);
-      wgetch (win[STA].p);
+      (void)wgetch (win[STA].p);
     }
-  else if (mode == IO_MODE_NONINTERACTIVE)
+  else if (ui_mode == UI_CMDLINE)
     {
       printf (proc_report, stats.lines);
       printf ("\n");
@@ -2448,43 +2651,91 @@ io_import_data (io_mode_t mode, import_type_t type, conf_t *conf,
   /* User has the choice to look at the log file if some items could not be
      imported.
   */
-  fclose (logfd);
+  fclose (log->fd);
   if (stats.skipped > 0)
     {
       char *view_log = _("Some items could not be imported, see log file ?");
-      char *choices = "[y/n] ";
-      int ans;
-      
-      if (mode == IO_MODE_NONINTERACTIVE)
-        {
-          int ans;
 
-          printf ("\n%s %s", view_log, choices);
-          ans = fgetc (stdin);
-          if (ans == 'y')
-            {
-              char cmd[BUFSIZ];
+      io_log_display (log, view_log, conf->pager);
+    }
+  io_log_free (log);
+}
 
-              snprintf (cmd, BUFSIZ, "%s %s", conf->pager, flogname);
-              system (cmd);
-            }
-        }
-      else
+io_file_t *
+io_log_init (void)
+{
+  const char *logprefix = "/tmp/calcurse_log.";
+  char *logname;
+  io_file_t *log;
+
+  logname = new_tempfile (logprefix, NOTESIZ);
+  RETVAL_IF (logname == 0, 0,
+             _("Warning: could not create temporary log file, Aborting..."));
+  log = malloc (sizeof (io_file_t));
+  RETVAL_IF (log == 0, 0,
+             _("Warning: could not open temporary log file, Aborting..."));
+  snprintf (log->name, sizeof (log->name), "%s%s", logprefix, logname);
+  mem_free (logname);
+  log->fd = fopen (log->name, "w");
+  if (log->fd == 0)
+    {
+      ERROR_MSG (_("Warning: could not open temporary log file, Aborting..."));
+      mem_free (log);
+      return 0;
+    }
+
+  return log;
+}
+
+void
+io_log_print (io_file_t *log, int line, char *msg)
+{
+  if (log && log->fd)
+    fprintf (log->fd, "line %d: %s\n", line, msg);
+}
+
+void
+io_log_display (io_file_t *log, char *msg, char *pager)
+{
+  char *choices = "[y/n] ";
+  int ans;
+
+  RETURN_IF (log == 0, _("No log file to display!"));
+  if (ui_mode == UI_CMDLINE)
+    {
+      printf ("\n%s %s", msg, choices);
+      ans = fgetc (stdin);
+      if (ans == 'y')
         {
-          status_mesg (view_log, choices);
-          do
-            {
-              ans = wgetch (win[STA].p);
-              if (ans == 'y')
-                {
-                  wins_launch_external (flogname, conf->pager);
-                }
-            }
-          while (ans != 'y' && ans != 'n');
-          erase_status_bar ();
+          char cmd[BUFSIZ];
+          
+          snprintf (cmd, BUFSIZ, "%s %s", pager, log->name);
+          system (cmd);
         }
     }
-  EXIT_IF (unlink (flogname) != 0,
-           _("Warning: could not erase temporary log file, Aborting..."));
-  mem_free (logname);
+  else
+    {
+      status_mesg (msg, choices);
+      do
+        {
+          ans = wgetch (win[STA].p);
+          if (ans == 'y')
+            {
+              wins_launch_external (log->name, pager);
+            }
+        }
+      while (ans != 'y' && ans != 'n');
+      erase_status_bar ();
+    }
+}
+
+void
+io_log_free (io_file_t *log)
+{
+  if (!log)
+    return;
+  EXIT_IF (unlink (log->name) != 0,
+           _("Warning: could not erase temporary log file %s, Aborting..."),
+           log->name);
+  mem_free (log);
 }
