@@ -1,4 +1,4 @@
-/*	$calcurse: notify.c,v 1.27 2008/04/20 13:49:39 culot Exp $	*/
+/*	$calcurse: notify.c,v 1.32 2008/12/18 20:38:52 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -32,10 +32,12 @@
 #include "i18n.h"
 #include "utils.h"
 #include "custom.h"
+#include "keys.h"
 #include "notify.h"
 
 static struct notify_vars_s *notify = NULL;
 static struct notify_app_s *notify_app = NULL;
+static pthread_attr_t detached_thread_attr;
 static pthread_t notify_t_main;
 
 /* Return 1 if we need to display the notify-bar, else 0. */
@@ -69,6 +71,10 @@ notify_init_vars (void)
 
   if ((nbar->shell = getenv ("SHELL")) == NULL)
     nbar->shell = "/bin/sh";
+
+  (void)pthread_attr_init (&detached_thread_attr);
+  (void)pthread_attr_setdetachstate (&detached_thread_attr,
+                                     PTHREAD_CREATE_DETACHED);
 }
 
 /* Extract the appointment file name from the complete file path. */
@@ -132,11 +138,10 @@ launch_cmd (char *cmd, char *shell)
   pid = fork ();
 
   if (pid < 0)
-    ierror (_("FATAL ERROR in launch_cmd: could not fork"), IERROR_WARN);
+    ERROR_MSG (_("error while launching command: could not fork"));
   else if (pid == 0)		/* Child: launch user defined command */
-    if (execlp (shell, shell, "-c", cmd, (char *) NULL) < 0)
-      ierror (_("FATAL ERROR in launch_cmd: could not "
-		"launch user command"), IERROR_WARN);
+    if (execlp (shell, shell, "-c", cmd, (char *)0) < 0)
+      ERROR_MSG (_("error while launching command"));
 }
 
 /* 
@@ -285,7 +290,7 @@ notify_thread_app (void *arg)
     {
       notify_app->got_app = 1;
       notify_app->time = tmp_app.time;
-      notify_app->txt = mycpy (tmp_app.txt);
+      notify_app->txt = strdup (tmp_app.txt);
       notify_app->state = tmp_app.state;
     }
   else
@@ -307,7 +312,8 @@ notify_check_next_app (void)
 {
   pthread_t notify_t_app;
 
-  pthread_create (&notify_t_app, NULL, notify_thread_app, NULL);
+  pthread_create (&notify_t_app, &detached_thread_attr, notify_thread_app,
+                  (void *)0);
   return;
 }
 
@@ -338,7 +344,7 @@ notify_check_added (char *mesg, long start, char state)
     {
       notify_app->got_app = 1;
       notify_app->time = start;
-      notify_app->txt = mycpy (mesg);
+      notify_app->txt = strdup (mesg);
       notify_app->state = state;
     }
   pthread_mutex_unlock (&notify_app->mutex);
@@ -377,7 +383,7 @@ notify_check_repeated (recur_apoint_llist_node_t *i)
     {
       notify_app->got_app = 1;
       notify_app->time = real_app_time;
-      notify_app->txt = mycpy (i->mesg);
+      notify_app->txt = strdup (i->mesg);
       notify_app->state = i->state;
     }
   pthread_mutex_unlock (&notify_app->mutex);
@@ -478,7 +484,7 @@ notify_print_options (WINDOW *optwin, int col)
   x = x_pos + x_offset + l;
   mvwprintw (optwin, y_pos, x_pos, "[1] %s", opt[SHOW].name);
   erase_window_part (optwin, x, y_pos, maxcol, y_pos);
-  print_option_incolor (optwin, nbar->show, y_pos, x);
+  print_bool_option_incolor (optwin, nbar->show, y_pos, x);
   mvwprintw (optwin, y_pos + 1, x_pos, opt[SHOW].desc);
 
   for (i = 1; i < NB_OPT; i++)
