@@ -1,4 +1,4 @@
-/*	$calcurse: recur.c,v 1.38 2008/09/16 19:41:36 culot Exp $	*/
+/*	$calcurse: recur.c,v 1.45 2008/12/14 15:54:51 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -34,6 +34,7 @@
 #include "utils.h"
 #include "notify.h"
 #include "day.h"
+#include "keys.h"
 #include "recur.h"
 
 recur_apoint_llist_t *recur_alist_p;
@@ -126,7 +127,6 @@ char
 recur_def2char (recur_types_t define)
 {
   char recur_char;
-  char *error = _("FATAL ERROR in recur_def2char: unknown recur type\n");
 
   switch (define)
     {
@@ -143,8 +143,8 @@ recur_def2char (recur_types_t define)
       recur_char = 'Y';
       break;
     default:
-      fputs (error, stderr);
-      exit (EXIT_FAILURE);
+      EXIT (_("unknown repetition type"));
+      return 0;
     }
 
   return (recur_char);
@@ -158,7 +158,6 @@ int
 recur_char2def (char type)
 {
   int recur_def;
-  char *error = _("FATAL ERROR in recur_char2def: unknown char\n");
 
   switch (type)
     {
@@ -175,8 +174,8 @@ recur_char2def (char type)
       recur_def = RECUR_YEARLY;
       break;
     default:
-      fputs (error, stderr);
-      exit (EXIT_FAILURE);
+      EXIT (_("unknown character"));
+      return 0;
     }
   return (recur_def);
 }
@@ -244,13 +243,8 @@ recur_apoint_scan (FILE *f, struct tm start, struct tm end, char type,
     {
       tuntil = 0;
     }
-
-  if (tstart == -1 || tend == -1 || tstart > tend || tuntil == -1)
-    {
-      fputs (_("FATAL ERROR in apoint_scan: date error in the appointment\n"),
-	     stderr);
-      exit (EXIT_FAILURE);
-    }
+  EXIT_IF (tstart == -1 || tend == -1 || tstart > tend || tuntil == -1,
+           _("date error in appointment"));
 
   return (recur_apoint_new (buf, note, tstart, tend - tstart, state,
 			    recur_char2def (type), freq, tuntil, exc));
@@ -292,12 +286,8 @@ recur_event_scan (FILE *f, struct tm start, int id, char type, int freq,
       tuntil = 0;
     }
   tstart = mktime (&start);
-  if ((tstart == -1) || (tuntil == -1))
-    {
-      fputs (_("FATAL ERROR in recur_event_scan: "
-	       "date error in the event\n"), stderr);
-      exit (EXIT_FAILURE);
-    }
+  EXIT_IF (tstart == -1 || tuntil == -1,
+           _("date error in event"));
 
   return (recur_event_new (buf, note, tstart, id, recur_char2def (type),
                            freq, tuntil, exc));
@@ -402,28 +392,11 @@ recur_save_data (FILE *f)
   pthread_mutex_unlock (&(recur_alist_p->mutex));
 }
 
-/* Given a day as long, check if this day belongs to the list of exceptions for
- * the considered item.
- */
-int
-recur_day_is_exc (long day, struct days_s *item_exc)
-{
-  const int NOT_EXC = 0;
-  const int EXC = 1;
-  struct days_s *exc;
-
-  for (exc = item_exc; exc != 0; exc = exc->next)
-    {
-      if (exc->st == day)
-        return EXC;
-    }
-  return NOT_EXC;
-}
-
 /* 
  * Check if the recurrent item belongs to the selected day,
  * and if yes, return the real start time.
  * This function was improved thanks to Tony's patch.
+ * Thanks also to youshe for reporting daylight saving time related problems.
  */
 unsigned
 recur_item_inday (long item_start, struct days_s *item_exc, int rpt_type,
@@ -434,12 +407,11 @@ recur_item_inday (long item_start, struct days_s *item_exc, int rpt_type,
   struct tm lt_item, lt_day;
   struct days_s *exc;
   time_t t;
-  char *error = _("FATAL ERROR in recur_item_inday: unknown item type\n");
 
   day_end = day_start + DAYINSEC;
   t = day_start;
   lt_day = *localtime (&t);
-
+  
   for (exc = item_exc; exc != 0; exc = exc->next)
     if (exc->st < day_end && exc->st >= day_start)
       return (0);
@@ -452,16 +424,12 @@ recur_item_inday (long item_start, struct days_s *item_exc, int rpt_type,
 
   t = item_start;
   lt_item = *localtime (&t);
-
-  /* For proper calculation, both items must start at same time. */
-  day_start += (lt_item.tm_hour * HOURINSEC + lt_item.tm_min * MININSEC +
-		lt_item.tm_sec);
-
+  
   switch (rpt_type)
     {
     case RECUR_DAILY:
-      diff = (long) difftime ((time_t) day_start, (time_t) item_start);
-      if (diff % (rpt_freq * DAYINSEC) != 0)
+      diff = lt_day.tm_yday - lt_item.tm_yday;
+      if (diff % rpt_freq != 0)
 	return (0);
       lt_item.tm_mday = lt_day.tm_mday;
       lt_item.tm_mon = lt_day.tm_mon;
@@ -495,8 +463,7 @@ recur_item_inday (long item_start, struct days_s *item_exc, int rpt_type,
       lt_item.tm_year = lt_day.tm_year;
       break;
     default:
-      fputs (error, stderr);
-      exit (EXIT_FAILURE);
+      EXIT (_("unknown item type"));
     }
   start_date.dd = lt_item.tm_mday;
   start_date.mm = lt_item.tm_mon + 1;
@@ -566,9 +533,8 @@ recur_event_erase (long start, unsigned num, unsigned delete_whole,
 	}
       iptr = &i->next;
     }
-  /* NOTREACHED */
-  fputs (_("FATAL ERROR in recur_event_erase: no such event\n"), stderr);
-  exit (EXIT_FAILURE);
+  EXIT (_("event not found"));
+  /* NOTREACHED */  
 }
 
 /*
@@ -638,10 +604,8 @@ recur_apoint_erase (long start, unsigned num, unsigned delete_whole,
 	}
       iptr = &i->next;
     }
+  EXIT (_("appointment not found"));
   /* NOTREACHED */
-  fputs (_("FATAL ERROR in recur_apoint_erase: no such appointment\n"),
-	 stderr);
-  exit (EXIT_FAILURE);
 }
 
 /*
@@ -718,7 +682,7 @@ recur_repeat_item (conf_t *conf)
 	  if (freq == 0)
 	    {
 	      status_mesg (mesg_wrong_freq, wrong_type_2);
-	      wgetch (win[STA].p);
+	      (void)wgetch (win[STA].p);
 	    }
 	  user_input[0] = '\0';
 	}
@@ -752,7 +716,7 @@ recur_repeat_item (conf_t *conf)
 		  if (until < p->start)
 		    {
 		      status_mesg (mesg_older, wrong_type_2);
-		      wgetch (win[STA].p);
+		      (void)wgetch (win[STA].p);
 		      date_entered = 0;
 		    }
 		  else
@@ -765,7 +729,7 @@ recur_repeat_item (conf_t *conf)
 		  snprintf (outstr, BUFSIZ, mesg_wrong_2,
 			    DATEFMT_DESC (conf->input_datefmt));
 		  status_mesg (mesg_wrong_1, _(outstr));
-		  wgetch (win[STA].p);
+		  (void)wgetch (win[STA].p);
 		  date_entered = 0;
 		}
 	    }
@@ -788,10 +752,9 @@ recur_repeat_item (conf_t *conf)
 	notify_check_repeated (ra);
     }
   else
-    {				/* NOTREACHED */
-      fputs (_("FATAL ERROR in recur_repeat_item: wrong item type\n"),
-	     stderr);
-      exit (EXIT_FAILURE);
+    {
+      EXIT (_("wrong item type"));
+      /* NOTREACHED */
     }
   day_erase_item (date, item_nb, ERASE_FORCE_KEEP_NOTE);
 }
@@ -818,9 +781,7 @@ recur_exc_scan (FILE *data_file)
       if (fscanf (data_file, "!%u / %u / %u ",
 		  &day.tm_mon, &day.tm_mday, &day.tm_year) != 3)
 	{
-	  fputs (_("FATAL ERROR in recur_exc_scan: "
-		   "syntax error in the item date\n"), stderr);
-	  exit (EXIT_FAILURE);
+          EXIT (_("syntax error in item date"));
 	}
       day.tm_sec = 0;
       day.tm_isdst = -1;
@@ -860,7 +821,7 @@ recur_apoint_check_next (struct notify_app_s *app, long start, long day)
 	  if (real_recur_start_time > start)
 	    {
 	      app->time = real_recur_start_time;
-	      app->txt = mycpy (i->mesg);
+	      app->txt = strdup (i->mesg);
 	      app->state = i->state;
 	      app->got_app = 1;
 	    }
@@ -892,9 +853,9 @@ recur_get_apoint (long date, int num)
 	  n++;
 	}
     }
+  EXIT (_("item not found"));
+  return 0;
   /* NOTREACHED */
-  fputs (_("FATAL ERROR in recur_get_apoint: no such item\n"), stderr);
-  exit (EXIT_FAILURE);
 }
 
 /* Returns a structure containing the selected recurrent event. */
@@ -916,9 +877,9 @@ recur_get_event (long date, int num)
 	  n++;
 	}
     }
+  EXIT (_("item not found"));
+  return 0;
   /* NOTREACHED */
-  fputs (_("FATAL ERROR in recur_get_event: no such item\n"), stderr);
-  exit (EXIT_FAILURE);
 }
 
 /* Switch recurrent item notification state. */
@@ -952,9 +913,7 @@ recur_apoint_switch_notify (long date, int recur_nb)
 	  n++;
 	}
     }
+  EXIT (_("item not found"));
   /* NOTREACHED */
-  fputs (_("FATAL ERROR in recur_apoint_switch_notify: no such item\n"),
-	 stderr);
-  exit (EXIT_FAILURE);
 }
 
