@@ -1,4 +1,4 @@
-/*	$calcurse: todo.c,v 1.22 2008/04/19 21:05:15 culot Exp $	*/
+/*	$calcurse: todo.c,v 1.28 2008/12/15 20:02:00 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -30,6 +30,7 @@
 
 #include "utils.h"
 #include "custom.h"
+#include "keys.h"
 #include "i18n.h"
 #include "todo.h"
 
@@ -189,8 +190,7 @@ todo_delete_note_bynum (unsigned num)
       if (n == num)
 	{
 	  if (i->note == NULL)
-	    ierror (_("FATAL ERROR in todo_delete_note_bynum: "
-		      "no note attached\n"), IERROR_FATAL);
+            EXIT (_("no note attached"));
 	  erase_note (&i->note, ERASE_FORCE_ONLY_NOTE);
 	  return;
 	}
@@ -198,9 +198,7 @@ todo_delete_note_bynum (unsigned num)
       n++;
     }
   /* NOTREACHED */
-  ierror (_("FATAL ERROR in todo_delete_note_bynum: no such todo\n"),
-	  IERROR_FATAL);
-  exit (EXIT_FAILURE);
+  EXIT (_("no such todo"));
 }
 
 /* Delete an item from the todo linked list. */
@@ -227,9 +225,7 @@ todo_delete_bynum (unsigned num, erase_flag_e flag)
       n++;
     }
   /* NOTREACHED */
-  ierror (_("FATAL ERROR in todo_delete_bynum: no such todo\n"),
-	  IERROR_FATAL);
-  exit (EXIT_FAILURE);
+  EXIT (_("no such todo"));
 }
 
 /* Delete an item from the ToDo list. */
@@ -243,7 +239,7 @@ todo_delete (conf_t *conf)
 	"Delete (t)odo or just its (n)ote ?");
   char *erase_choice = _("[t/n] ");
   bool go_for_todo_del = false;
-  int answer = 0, has_note;
+  int answer, has_note;
 
   if (conf->confirm_delete)
     {
@@ -268,12 +264,12 @@ todo_delete (conf_t *conf)
       return;
     }
 
-  answer = 0;
+  answer = -1;
   has_note = (todo_get_item (hilt)->note != NULL) ? 1 : 0;
   if (has_note == 0)
     answer = 't';
 
-  while (answer != 't' && answer != 'n' && answer != ESCAPE)
+  while (answer != 't' && answer != 'n' && answer != KEY_GENERIC_CANCEL)
     {
       status_mesg (erase_warning, erase_choice);
       answer = wgetch (win[STA].p);
@@ -325,8 +321,8 @@ todo_get_position (struct todo_s *i)
     }
   else
     {
-      fputs (_("FATAL ERROR in todo_get_position: todo not found\n"), stderr);
-      exit (EXIT_FAILURE);
+      EXIT (_("todo not found"));
+      return -1; /* avoid compiler warnings */
     }
 }
 
@@ -347,17 +343,17 @@ todo_chg_priority (int action)
     strncpy (backup_note, backup->note, NOTESIZ + 1);
   else
     backup_note[0] = '\0';
-  if (action == '+')
+  switch (action)
     {
+    case KEY_RAISE_PRIORITY:
       (backup_id > 1) ? backup_id-- : do_chg--;
-    }
-  else if (action == '-')
-    {
+      break;
+    case KEY_LOWER_PRIORITY:
       (backup_id < 9) ? backup_id++ : do_chg--;
-    }
-  else
-    {				/* NOTREACHED */
-      fputs (_("FATAL ERROR in todo_chg_priority: no such action\n"), stderr);
+      break;
+    default:
+      EXIT (_("no such action"));
+      /* NOTREACHED */
     }
   if (do_chg)
     {
