@@ -1,4 +1,4 @@
-/*	$calcurse: utils.c,v 1.52 2008/09/23 17:31:57 culot Exp $	*/
+/*	$calcurse: utils.c,v 1.61 2008/12/20 19:27:31 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -37,8 +37,15 @@
 #include "i18n.h"
 #include "wins.h"
 #include "custom.h"
+#include "keys.h"
 #include "utils.h"
 
+#define NB_CAL_CMDS	24	/* number of commands while in cal view */
+#define NB_APP_CMDS	29	/* same thing while in appointment view */
+#define NB_TOD_CMDS	29	/* same thing while in todo view */
+#define TOTAL_CMDS	NB_CAL_CMDS + NB_APP_CMDS + NB_TOD_CMDS
+#define CMDS_PER_LINE	6	/* max number of commands per line */  
+
 static unsigned status_page;
 
 /* General routine to exit calcurse properly. */
@@ -55,67 +62,27 @@ exit_calcurse (int status)
 
 /* Function to exit on internal error. */
 void
-ierror (const char *errmsg, ierror_sev_e sev)
+fatalbox (const char *errmsg)
 {
   WINDOW *errwin;
-  char *label = _("INTERNAL ERROR");
-  char *exitmsg = _("calcurse will now exit...");
-  char *reportmsg = _("Please report the following bug:");
-  const int winrow = 10;
-  const int wincol = col - 2;
-  const int msglen = wincol - 2;
-  char msg[msglen];
+  char *label = _("/!\\ INTERNAL ERROR /!\\");
+    char *reportmsg = _("Please report the following bug:");
+  const int WINROW = 10;
+  const int WINCOL = col - 2;
+  const int MSGLEN = WINCOL - 2;
+  char msg[MSGLEN];
 
-  strncpy (msg, errmsg, msglen);
-  errwin = newwin (winrow, wincol, (row - winrow) / 2, (col - wincol) / 2);
+  strncpy (msg, errmsg, MSGLEN);
+  errwin = newwin (WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2);
   custom_apply_attr (errwin, ATTR_HIGHEST);
   box (errwin, 0, 0);
   wins_show (errwin, label);
-  if (sev == IERROR_FATAL)
-    mvwprintw (errwin, 3, 1, reportmsg);
-  mvwprintw (errwin, 5, (wincol - strlen (msg)) / 2, "%s", msg);
-  if (sev == IERROR_FATAL)
-    mvwprintw (errwin, winrow - 2, wincol - strlen (exitmsg) - 1, "%s",
-	       exitmsg);
+  mvwprintw (errwin, 3, 1, reportmsg);
+  mvwprintw (errwin, 5, (WINCOL - strlen (msg)) / 2, "%s", msg);
   custom_remove_attr (errwin, ATTR_HIGHEST);
   wrefresh (errwin);
-  wgetch (errwin);
-  if (sev == IERROR_FATAL)
-    exit_calcurse (EXIT_FAILURE);
-}
-
-/* Function to handle an assertion failure. */
-void
-aerror (const char *file, int line, const char *assertion)
-{
-  char errmsg[BUFSIZ];
-
-  snprintf (errmsg, BUFSIZ,
-	    "assert \"%s\" failed: file \"%s\", line %d", assertion, file,
-	    line);
-  ierror (errmsg, IERROR_FATAL);
-}
-
-void
-warnbox (const char *msg)
-{
-  const int WINCOL = col - 2;
-  const int WINROW = 10;
-  const int MSGLEN = WINCOL - 2;
-  WINDOW *warnwin;
-  char displmsg[MSGLEN];
-  
-  if (msg == NULL)
-    return;
-  strncpy (displmsg, msg, MSGLEN);
-  warnwin = popup (WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2,
-                   "/!\\");
-  custom_apply_attr (warnwin, ATTR_HIGHEST);
-  mvwprintw (warnwin, 5, (WINCOL - strlen (displmsg)) / 2, "%s", displmsg);
-  custom_remove_attr (warnwin, ATTR_HIGHEST);
-  wrefresh (warnwin);
-  wgetch (warnwin);
-  delwin (warnwin);
+  (void)wgetch (errwin);
+  delwin (errwin);
   doupdate ();
 }
 
@@ -159,24 +126,30 @@ erase_window_part (WINDOW *win, int first_col, int first_row, int last_col,
 
 /* draws a popup window */
 WINDOW *
-popup (int pop_row, int pop_col, int pop_y, int pop_x, char *pop_lab)
+popup (int pop_row, int pop_col, int pop_y, int pop_x, char *title, char *msg,
+       int hint)
 {
-  char *txt_pop = _("Press any key to continue...");
+  char *any_key = _("Press any key to continue...");
   char label[BUFSIZ];
   WINDOW *popup_win;
+  const int MSGXPOS = 5;
 
   popup_win = newwin (pop_row, pop_col, pop_y, pop_x);
   keypad (popup_win, TRUE);
+  if (msg)
+    mvwprintw (popup_win, MSGXPOS, (pop_col - strlen (msg)) / 2, "%s", msg);
   custom_apply_attr (popup_win, ATTR_HIGHEST);
   box (popup_win, 0, 0);
-  snprintf (label, BUFSIZ, "%s", pop_lab);
+  snprintf (label, BUFSIZ, "%s", title);
   wins_show (popup_win, label);
-  mvwprintw (popup_win, pop_row - 2, pop_col - (strlen (txt_pop) + 1), "%s",
-	     txt_pop);
+  if (hint)
+    mvwprintw (popup_win, pop_row - 2, pop_col - (strlen (any_key) + 1), "%s",
+               any_key);
   custom_remove_attr (popup_win, ATTR_HIGHEST);
-  wnoutrefresh (popup_win);
+  wrefresh (popup_win);
   doupdate ();
-  return (popup_win);
+  
+  return popup_win;
 }
 
 /* prints in middle of a panel */
@@ -216,7 +189,7 @@ showcursor (WINDOW *win, int y, int pos, char *str, int l, int offset)
 
   nc = str + pos;
   wmove (win, y, pos - offset);
-  (pos >= l) ? waddch (win, SPC | A_REVERSE) : waddch (win, *nc | A_REVERSE);
+  (pos >= l) ? waddch (win, SPACE | A_REVERSE) : waddch (win, *nc | A_REVERSE);
 }
 
 /* Print the string at the desired position. */
@@ -356,7 +329,7 @@ getstring (WINDOW *win, char *str, int l, int x, int y)
 	    newpos++;
 	  break;
 
-	case ESCAPE:		/* cancel editing */
+	case ESCAPE:	/* cancel editing */
 	  return (GETSTRING_ESC);
 	  break;
 
@@ -396,14 +369,9 @@ updatestring (WINDOW *win, char **str, int x, int y)
   if (!escape)
     {
       len = strlen (newstr) + 1;
-      if ((*str = (char *) realloc (*str, len)) == NULL)
-	{
-	  /* NOTREACHED */
-	  ierror (_("FATAL ERROR in updatestring: out of memory"),
-		  IERROR_FATAL);
-	}
-      else
-	(void) memcpy (*str, newstr, len);
+      *str = (char *) realloc (*str, len);
+      EXIT_IF (*str == 0, _("out of memory"));
+      (void) memcpy (*str, newstr, len);
     }
   free (newstr);
   return (escape);
@@ -430,86 +398,72 @@ is_all_digit (char *string)
 /* 
  * Draws the status bar. 
  * To add a keybinding, insert a new binding_t item, add it in the *binding
- * table, and update the NB_CAL_CMDS, NB_APP_CMDS or NB_TOD_CMDS defines in
- * utils.h, depending on which panel the added keybind is assigned to.
+ * table, and update the NB_CAL_CMDS, NB_APP_CMDS or NB_TOD_CMDS defines,
+ * depending on which panel the added keybind is assigned to.
  */
 void
 status_bar (void)
 {
+#define NB_PANELS	3	/* 3 panels: CALENDAR, APPOINTMENT, TODO */
   window_e which_pan;
-  int cmd_length, space_between_cmds, start, end, i, j = 0;
+  int start, end;
   const int pos[NB_PANELS + 1] =
       { 0, NB_CAL_CMDS, NB_CAL_CMDS + NB_APP_CMDS, TOTAL_CMDS };
 
-  binding_t help = { "  ?", _("Help") };
-  binding_t quit = { "  Q", _("Quit") };
-  binding_t save = { "  S", _("Save") };
-  binding_t export = { "  X", _("Export") };
-  binding_t import = { "  I", _("Import") };  
-  binding_t add = { "  A", _("Add Item") };
-  binding_t del = { "  D", _("Del Item") };
-  binding_t edit = { "  E", _("Edit Itm") };
-  binding_t flag = { "  !", _("Flag Itm") };
-  binding_t day = { "H/L", _("-+1 Day") };
-  binding_t week = { "K/J", _("-+1 Week") };
-  binding_t updn = { "K/J", _("Up/Down") };
-  binding_t rept = { "  R", _("Repeat") };
-  binding_t prio = { "+/-", _("Priority") };
-  binding_t tab = { "Tab", _("Chg View") };
-  binding_t togo = { "  G", _("Go to") };
-  binding_t conf = { "  C", _("Config") };
-  binding_t view = { "  V", _("View") };
-  binding_t draw = { " ^R", _("Redraw") };
-  binding_t appt = { " ^A", _("Add Appt") };
-  binding_t todo = { " ^T", _("Add Todo") };
-  binding_t enote = { "  N", _("EditNote") };
-  binding_t vnote = { "  >", _("ViewNote") };
-  binding_t eday = { "^HL", _("-+1 Day") };
-  binding_t ewek = { "^KJ", _("-+1 Week") };
-  binding_t othr = { "  O", _("OtherCmd") };
-  binding_t today = {" ^G", _("Today") };
-  binding_t weekb = {"  0", _("beg Week") };
-  binding_t weeke = {"  $", _("end Week") };
-
+  binding_t help   = {_("Help"),     KEY_GENERIC_HELP};
+  binding_t quit   = {_("Quit"),     KEY_GENERIC_QUIT};
+  binding_t save   = {_("Save"),     KEY_GENERIC_SAVE};
+  binding_t chgvu  = {_("Chg View"), KEY_GENERIC_CHANGE_VIEW};
+  binding_t import = {_("Import"),   KEY_GENERIC_IMPORT};  
+  binding_t export = {_("Export"),   KEY_GENERIC_EXPORT};
+  binding_t togo   = {_("Go to"),    KEY_GENERIC_GOTO};
+  binding_t othr   = {_("OtherCmd"), KEY_GENERIC_OTHER_CMD};
+  binding_t conf   = {_("Config"),   KEY_GENERIC_CONFIG_MENU};  
+  binding_t draw   = {_("Redraw"),   KEY_GENERIC_REDRAW};
+  binding_t appt   = {_("Add Appt"), KEY_GENERIC_ADD_APPT};
+  binding_t todo   = {_("Add Todo"), KEY_GENERIC_ADD_TODO};
+  binding_t gnday  = {_("+1 Day"),   KEY_GENERIC_NEXT_DAY};
+  binding_t gpday  = {_("-1 Day"),   KEY_GENERIC_PREV_DAY};
+  binding_t gnweek = {_("+1 Week"),  KEY_GENERIC_NEXT_WEEK};
+  binding_t gpweek = {_("-1 Week"),  KEY_GENERIC_PREV_WEEK};
+  binding_t today  = {_("Today"),    KEY_GENERIC_GOTO_TODAY};
+  binding_t up     = {_("Up"),       KEY_MOVE_UP};
+  binding_t down   = {_("Down"),     KEY_MOVE_DOWN};  
+  binding_t left   = {_("Left"),     KEY_MOVE_LEFT};
+  binding_t right  = {_("Right"),    KEY_MOVE_RIGHT};  
+  binding_t weekb  = {_("beg Week"), KEY_START_OF_WEEK};
+  binding_t weeke  = {_("end Week"), KEY_END_OF_WEEK};
+  binding_t add    = {_("Add Item"), KEY_ADD_ITEM};
+  binding_t del    = {_("Del Item"), KEY_DEL_ITEM};
+  binding_t edit   = {_("Edit Itm"), KEY_EDIT_ITEM};
+  binding_t view   = {_("View"),     KEY_VIEW_ITEM};  
+  binding_t flag   = {_("Flag Itm"), KEY_FLAG_ITEM};
+  binding_t rept   = {_("Repeat"),   KEY_REPEAT_ITEM};
+  binding_t enote  = {_("EditNote"), KEY_EDIT_NOTE};
+  binding_t vnote  = {_("ViewNote"), KEY_VIEW_NOTE};
+  binding_t rprio  = {_("Prio.+"),   KEY_RAISE_PRIORITY};
+  binding_t lprio  = {_("Prio.-"),   KEY_LOWER_PRIORITY};  
+  
   binding_t *binding[TOTAL_CMDS] = {
     /* calendar keys */
-    &help, &quit, &save, &tab, &import, &export, &day, &week, &weekb, &weeke,
-    &togo, &othr, &conf, &draw, &appt, &todo, &eday, &ewek, &today, &othr,
+    &help, &quit, &save, &chgvu, &import, &export, &up, &down, &left, &right,
+    &togo, &othr, &weekb, &weeke, &conf, &draw, &appt, &todo, &gnday, &gpday,
+    &gnweek, &gpweek, &today, &othr,
     /* appointment keys */
-    &help, &quit, &save, &tab, &import, &export, &add, &del, &edit, &view,
-    &rept, &othr, &updn, &flag, &enote, &vnote, &appt, &todo, &eday, &ewek,
-    &conf, &togo, &draw, &othr, &today, &othr,
+    &help, &quit, &save, &chgvu, &import, &export, &add, &del, &edit, &view,
+    &draw, &othr, &rept, &flag, &enote, &vnote, &up, &down, &gnday, &gpday,
+    &gnweek, &gpweek, &togo, &othr, &today, &conf, &appt, &todo, &othr,
     /* todo keys */
-    &help, &quit, &save, &tab, &import, &export, &add, &del, &edit, &view,
-    &prio, &othr, &updn, &conf, &enote, &vnote, &appt, &todo, &eday, &ewek,
-    &togo, &draw, &today, &othr
+    &help, &quit, &save, &chgvu, &import, &export, &add, &del, &edit, &view,
+    &draw, &othr, &rprio, &lprio, &enote, &vnote, &up, &down, &gnday, &gpday,
+    &gnweek, &gpweek, &togo, &othr, &today, &conf, &appt, &todo, &othr
   };
 
-  /* Total length of a command. */
-  cmd_length = KEY_LENGTH + LABEL_LENGTH;
-  space_between_cmds = floor (col / CMDS_PER_LINE - cmd_length);
-  cmd_length += space_between_cmds;
-
   /* Drawing the keybinding with attribute and label without. */
-  erase_status_bar ();
   which_pan = wins_slctd ();
-  start = pos[which_pan] + 2 * CMDS_PER_LINE * (status_page - 1);
-  end = MIN (start + 2 * CMDS_PER_LINE, pos[which_pan + 1]);
-  for (i = start; i < end; i += 2)
-    {
-      custom_apply_attr (win[STA].p, ATTR_HIGHEST);
-      mvwprintw (win[STA].p, 0, j * cmd_length, binding[i]->key);
-      if (i + 1 != end)
-	mvwprintw (win[STA].p, 1, j * cmd_length, binding[i + 1]->key);
-      custom_remove_attr (win[STA].p, ATTR_HIGHEST);
-      mvwprintw (win[STA].p, 0, j * cmd_length + KEY_LENGTH,
-		 binding[i]->label);
-      if (i + 1 != end)
-	mvwprintw (win[STA].p, 1, j * cmd_length + KEY_LENGTH,
-		   binding[i + 1]->label);
-      j++;
-    }
-  wnoutrefresh (win[STA].p);
+  start = pos[which_pan] + 2 * KEYS_CMDS_PER_LINE * (status_page - 1);
+  end = MIN (start + 2 * KEYS_CMDS_PER_LINE, pos[which_pan + 1]);
+  keys_display_bindings_bar (win[STA].p, binding, start, end);
 }
 
 long
@@ -532,13 +486,7 @@ date2sec (date_t day, unsigned hour, unsigned min)
   start.tm_year -= 1900;
   start.tm_mon--;
   tstart = mktime (&start);
-  if (tstart == -1)
-    {
-      fputs (_("FATAL ERROR in date2sec: failure in mktime\n"), stderr);
-      fprintf (stderr, "%u %u %u %u %u\n", day.yyyy, day.mm, day.dd,
-	       hour, min);
-      exit (EXIT_FAILURE);
-    }
+  EXIT_IF (tstart == -1, _("failure in mktime"));
 
   return (tstart);
 }
@@ -606,12 +554,9 @@ date_sec_change (long date, int delta_month, int delta_day)
   lt = localtime (&t);
   lt->tm_mon += delta_month;
   lt->tm_mday += delta_day;
+  lt->tm_isdst = -1;
   t = mktime (lt);
-  if (t == -1)
-    {
-      fputs (_("FATAL ERROR in date_sec_change: failure in mktime\n"), stderr);
-      exit (EXIT_FAILURE);
-    }
+  EXIT_IF (t == -1, _("failure in mktime"));
 
   return t;
 }
@@ -631,7 +576,7 @@ update_time_in_date (long date, unsigned hr, unsigned mn)
   lt->tm_hour = hr;
   lt->tm_min = mn;
   new_date = mktime (lt);
-  ASSERT (new_date != -1);
+  EXIT_IF (new_date == -1, _("error in mktime"));
 
   return (new_date);
 }
@@ -749,7 +694,7 @@ item_in_popup (char *saved_a_start, char *saved_a_end, char *msg,
   const int padl = winl - 2, padw = winw - margin_left;
 
   pad = newpad (padl, padw);
-  popup_win = popup (winl, winw, 1, 2, pop_title);
+  popup_win = popup (winl, winw, 1, 2, pop_title, (char *)0, 1);
   if (strncmp (pop_title, _("Appointment"), 11) == 0)
     {
       mvwprintw (popup_win, margin_top, margin_left, "- %s -> %s",
@@ -759,7 +704,7 @@ item_in_popup (char *saved_a_start, char *saved_a_end, char *msg,
   wmove (win[STA].p, 0, 0);
   pnoutrefresh (pad, 0, 0, margin_top + 2, margin_left, padl, winw);
   doupdate ();
-  wgetch (popup_win);
+  (void)wgetch (popup_win);
   delwin (pad);
   delwin (popup_win);
 }
@@ -776,7 +721,6 @@ void
 other_status_page (int panel)
 {
   int nb_item = 0, max_page;
-  char *error = _("FATAL ERROR in other_status_page: unknown panel\n");
 
   switch (panel)
     {
@@ -790,7 +734,8 @@ other_status_page (int panel)
       nb_item = NB_TOD_CMDS;
       break;
     default:
-      ierror (error, IERROR_FATAL);
+      EXIT (_("unknown panel"));
+      /* NOTREACHED */
     }
   max_page = ceil (nb_item / (2 * CMDS_PER_LINE + 1)) + 1;
   if (status_page < max_page)
@@ -828,41 +773,25 @@ now (void)
   return (current_time);
 }
 
-/* Copy a string */
-char *
-mycpy (const char *src)
-{
-  char *string = malloc (strlen (src) + 1);
-
-  if (string != NULL)
-    return (strncpy (string, src, strlen (src) + 1));
-  else
-    return (NULL);
-}
-
 long
 mystrtol (const char *str)
 {
   char *ep;
   long lval;
-  const char *not_a_number =
-    _("FATAL ERROR in mystrtol: could not convert string");
-  const char *out_of_range =
-    _("FATAL ERROR in mystrtol: number is out of range");
 
   errno = 0;
   lval = strtol (str, &ep, 10);
   if (str[0] == '\0' || *ep != '\0')
-    ierror (not_a_number, IERROR_FATAL);
+    EXIT (_("could not convert string"));
   if (errno == ERANGE && (lval == LONG_MAX || lval == LONG_MIN))
-    ierror (out_of_range, IERROR_FATAL);
+    EXIT (_("out of range"));
 
   return (lval);
 }
 
 /* Print the given option value with appropriate color. */
 void
-print_option_incolor (WINDOW *win, bool option, int pos_y, int pos_x)
+print_bool_option_incolor (WINDOW *win, bool option, int pos_y, int pos_x)
 {
   int color = 0;
   char option_value[BUFSIZ] = "";
@@ -878,10 +807,8 @@ print_option_incolor (WINDOW *win, bool option, int pos_y, int pos_x)
       strncpy (option_value, _("no"), BUFSIZ);
     }
   else
-    {
-      ierror (_("option not defined - Problem in print_option_incolor()"),
-	      IERROR_FATAL);
-    }
+    EXIT (_("option not defined"));
+
   custom_apply_attr (win, color);
   mvwprintw (win, pos_y, pos_x, "%s", option_value);
   custom_remove_attr (win, color);
@@ -916,9 +843,8 @@ new_tempfile (const char *prefix, int trailing_len)
 	  unlink (fullname);
 	  close (fd);
 	}
-      ierror (_("FATAL ERROR: temporary file could not be created!"),
-	      IERROR_WARN);
-      return (NULL);
+      ERROR_MSG (_("temporary file could not be created"));
+      return (char *)0;
     }
   fclose (file);
 
@@ -930,7 +856,6 @@ void
 erase_note (char **note, erase_flag_e flag)
 {
   char fullname[BUFSIZ];
-  char *errmsg = _("FATAL ERROR in erase_note: could not remove note\n");
 
   if (*note == NULL)
     return;
@@ -938,7 +863,7 @@ erase_note (char **note, erase_flag_e flag)
     {
       snprintf (fullname, BUFSIZ, "%s%s", path_notes, *note);
       if (unlink (fullname) != 0)
-	ierror (errmsg, IERROR_FATAL);
+        EXIT (_("could not remove note"));
     }
   free (*note);
   *note = NULL;
@@ -1003,6 +928,8 @@ str_toupper (char *s)
 void
 mem_free (void *ptr)
 {
-  if (ptr)
-    free (ptr);
+  if (!ptr)
+    return;
+  free (ptr);
+  ptr = NULL;
 }
