@@ -1,4 +1,4 @@
-/*	$calcurse: utils.h,v 1.36 2008/09/23 17:31:57 culot Exp $	*/
+/*	$calcurse: utils.h,v 1.40 2008/12/12 20:44:50 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -27,19 +27,26 @@
 #ifndef CALCURSE_UTILS_H
 #define CALCURSE_UTILS_H
 
+#include <stdlib.h>
+
 #include "calendar.h"
+#include "vars.h"
 
 #define MAX(x,y) 	((x)>(y)?(x):(y))
 #define MIN(x,y) 	((x)<(y)?(x):(y))
 
+#define STRING_BUILD(str) {str, sizeof (str) - 1}
+
 #define ERROR_MSG(...) do {                                             \
-        char msg[BUFSIZ];                                               \
+  char msg[BUFSIZ];                                                     \
+  int len;                                                              \
                                                                         \
-        snprintf (msg, BUFSIZ, __VA_ARGS__);                            \
-        if (ui_mode == UI_CURSES)                                       \
-          warnbox (msg);                                                \
-        else                                                            \
-          fprintf (stderr, "%s\n", msg);                                \
+  len = snprintf (msg, BUFSIZ, "%s: %d: ", __FILE__, __LINE__);         \
+  snprintf (msg + len, BUFSIZ - len, __VA_ARGS__);                      \
+  if (ui_mode == UI_CURSES)                                             \
+    fatalbox (msg);                                                     \
+  else                                                                  \
+    fprintf (stderr, "%s\n", msg);                                      \
 } while (0)
 
 #define EXIT(...) do {                                                  \
@@ -51,64 +58,34 @@
 } while (0)
 
 #define EXIT_IF(cond, ...) do {                                         \
-        if ((cond))                                                     \
-          {                                                             \
-            ERROR_MSG(__VA_ARGS__);                                     \
-            if (ui_mode == UI_CURSES)                                   \
-              exit_calcurse (EXIT_FAILURE);                             \
-            else                                                        \
-              exit (EXIT_FAILURE);                                      \
-          }                                                             \
+  if ((cond))                                                           \
+    EXIT(__VA_ARGS__);                                                  \
 } while (0)
 
 #define RETURN_IF(cond, ...) do {                                       \
-        if ((cond))                                                     \
-          {                                                             \
-            ERROR_MSG(__VA_ARGS__);                                     \
-            return;                                                     \
-          }                                                             \
+  if ((cond))                                                           \
+    {                                                                   \
+      ERROR_MSG(__VA_ARGS__);                                           \
+      return;                                                           \
+    }                                                                   \
 } while (0)
 
 #define RETVAL_IF(cond, val, ...) do {                                  \
-        if ((cond))                                                     \
-          {                                                             \
-            ERROR_MSG(__VA_ARGS__);                                     \
-            return (val);                                               \
-          }                                                             \
+  if ((cond))                                                           \
+    {                                                                   \
+      ERROR_MSG(__VA_ARGS__);                                           \
+      return (val);                                                     \
+    }                                                                   \
 } while (0)
 
-#define ASSERT(e) do {							\
-	((e) ? (void)0 : aerror(__FILE__, __LINE__, #e));		\
-} while (0)
-
-#define SPC		32	/* ASCII code for white space */
-
-#define NB_CAL_CMDS	20	/* number of commands while in cal view */
-#define NB_APP_CMDS	26	/* same thing while in appointment view */
-#define NB_TOD_CMDS	24	/* same thing while in todo view */
-#define TOTAL_CMDS	NB_CAL_CMDS + NB_APP_CMDS + NB_TOD_CMDS
-#define NB_PANELS	3	/* 3 panels: CALENDAR, APPOINTMENT, TODO */
-#define CMDS_PER_LINE	6	/* max number of commands per line */
-#define KEY_LENGTH	4	/* length of each keybinding + one space */
-#define LABEL_LENGTH	8	/* length of command description */
-
 #define GETSTRING_VALID	0	/* value returned by getstring() if text is valid */
 #define GETSTRING_ESC	1	/* user pressed escape to cancel editing */
 #define GETSTRING_RET	2	/* return was pressed without entering any text */
 
-typedef struct
-{				/* structure defining a keybinding */
-  char *key;
-  char *label;
-}
-binding_t;
-
-typedef enum
-{
-  IERROR_FATAL,
-  IERROR_WARN
-}
-ierror_sev_e;
+typedef struct {
+  const char *str;
+  const int len;
+} string_t;
 
 typedef enum
 {
@@ -120,13 +97,11 @@ typedef enum
 erase_flag_e;
 
 void    exit_calcurse (int);
-void    ierror (const char *, ierror_sev_e);
-void    aerror (const char *, int, const char *);
-void    warnbox (const char *);
+void    fatalbox (const char *);
 void    status_mesg (char *, char *);
 void    erase_status_bar (void);
 void    erase_window_part (WINDOW *, int, int, int, int);
-WINDOW *popup (int, int, int, int, char *);
+WINDOW *popup (int, int, int, int, char *, char *, int);
 void    print_in_middle (WINDOW *, int, int, int, char *);
 int     getstring (WINDOW *, char *, int, int, int);
 int     updatestring (WINDOW *, char **, int, int);
@@ -147,9 +122,8 @@ void    reset_status_page (void);
 void    other_status_page (int);
 long    get_today (void);
 long    now (void);
-char   *mycpy (const char *);
 long    mystrtol (const char *);
-void    print_option_incolor (WINDOW *, bool, int, int);
+void    print_bool_option_incolor (WINDOW *, bool, int, int);
 char   *new_tempfile (const char *, int);
 void    erase_note (char **, erase_flag_e);
 int     parse_date (char *, int, int *, int *, int *);
