@@ -1,4 +1,4 @@
-/*	$calcurse: vars.c,v 1.9 2008/09/20 12:47:06 culot Exp $	*/
+/*	$calcurse: vars.c,v 1.11 2008/11/16 17:42:53 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -31,6 +31,7 @@
 #include "calendar.h"
 #include "custom.h"
 #include "wins.h"
+#include "keys.h"
 #include "vars.h"
 
 /*
@@ -87,6 +88,7 @@ char path_todo[] = "";
 char path_apts[] = "";
 char path_conf[] = "";
 char path_notes[] = "";
+char path_keys[] = "";
 
 /* Variable to handle pads. */
 struct pad_s *apad;
